# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import uuid

import mock

from openstackclient.tests.unit import utils

from otcextensions.tests.unit.osclient import test_base

from otcextensions.sdk.dcaas.v2 import connection
from otcextensions.sdk.dcaas.v2 import endpoint_group
from otcextensions.sdk.dcaas.v2 import virtual_gateway


def gen_data(data, columns):
    """Fill expected data tuple based on columns list
    """
    return tuple(getattr(data, attr, '') for attr in columns)


def gen_data_dict(data, columns):
    """Fill expected data tuple based on columns list
    """
    return tuple(data.get(attr, '') for attr in columns)


class TestDcaas(utils.TestCommand):
    def setUp(self):
        super(TestDcaas, self).setUp()
        self.app.client_manager.dcaas = mock.Mock()
        self.client = self.app.client_manager.dcaas


class FakeDirectConnection(test_base.Fake):
    """Fake one or more Direct Connection"""
    @classmethod
    def generate(cls):
        """Create a fake Direct Connection.

        :return:
            A FakeResource object, with id, name and so on
        """
        object_info = {
            "id": "id-" + uuid.uuid4().hex,
            "name": "name-" + uuid.uuid4().hex,
            "port_type": "1G",
            "bandwidth": 10,
            "provider": "OTC",
            "location": "Biere",
            "description": 'test description',
            "peer_location": 'test_peer_loc',
            "device_id": '172.16.40.2',
            "interface_name": 'Eth-Trunk2',
            "redundant_id": '11111',
            "provider_status": 'ACTIVE',
            "type": 'hosted',
            "hosting_id": 'test_11',
            "vlan": 11,
            "charge_mode": 'traffic',
            "order_id": 'id1',
            "product_id": 'idp1',
            "status": 'ACTIVE',
            "admin_state_up": True
        }
        return connection.Connection(**object_info)


class FakeEndpointGroup(test_base.Fake):
    """Fake one or more Endpoint Group"""
    @classmethod
    def generate(cls):
        """Create a fake Endpoint Group.

        :return:
            A FakeResource object, with id, name and so on
        """
        object_info = {
            "id": "id-" + uuid.uuid4().hex,
            "name": "name" + uuid.uuid4().hex,
            "description": "test description",
            "tenant_id": "tid" + uuid.uuid4().hex,
            "type": "cidr",
            "endpoints": ["10.2.0.0/24", "10.3.0.0/24"],
        }
        return endpoint_group.DirectConnectEndpointGroup(**object_info)


class FakeVirtualGateway(test_base.Fake):
    """Fake one or more Virtual Gateway."""
    @classmethod
    def generate(cls):
        """Create a fake Virtual Gateway.

        :return:
            A FakeResource object, with id, name and so on
        """
        object_info = {
            "id": "id-" + uuid.uuid4().hex,
            "name": "name-" + uuid.uuid4().hex,
            "description": "test description",
            "vpc_id": "908d9cf3-da64-4acb-393f-e5eb6b9e838a",
            "local_ep_group_id": "f8834cf1-5468-87c7-223d-56e78b9699ab",
            "device_id": "test_device_id1",
            "redundant_device_id": "test_r_device_id1",
            "type": "double ipsec",
            "bgp_asn": 10,
            "ipsec_bandwidth": 50,
            "status": "ACTIVE",
            "admin_state_up": True
        }
        return virtual_gateway.VirtualGateway(**object_info)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock

from otcextensions.osclient.modelartsv1.v1 import service_flavor
from otcextensions.tests.unit.osclient.modelartsv1.v1 import fakes


class TestSpecifications(fakes.TestModelartsv1):
    objects = fakes.FakeServiceSpecification.create_multiple(3)

    column_list_headers = (
        "Specification",
        "Billing Spec",
        "Spec Status",
        "Is Open",
        "Is Free",
        "Over Quota",
        "Display EN",
        "Extend Params",
    )

    data = []

    for s in objects:
        data.append(
            (
                s.specification,
                s.billing_spec,
                s.spec_status,
                s.is_open,
                s.is_free,
                s.over_quota,
                s.display_en,
                s.extend_params,
            )
        )

    def setUp(self):
        super(TestSpecifications, self).setUp()

        self.cmd = service_flavor.ListServiceFlavors(self.app, None)

        self.client.service_flavors = mock.Mock()
        self.client.api_mock = self.client.service_flavors

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            "--is-personal-cluster",
            "--infer-type",
            "real-time",
        ]

        verifylist = [
            ("is_personal_cluster", True),
            ("infer_type", "real-time"),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            is_personal_cluster=True,
            infer_type="real-time",
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class ReplicationCluster(resource.Resource):
    #: Properties
    #: Availability zone name
    availability_zone = resource.Body('availability_zone')


class Domains(resource.Resource):
    #: Properties
    #: Active-active domain id
    id = resource.Body('id')
    #: Active-active domain name
    name = resource.Body('name')
    #: Active-active domain description
    description = resource.Body('description')
    #: Active-active domain availability
    sold_out = resource.Body('sold_out', type=bool)
    #: Active-active domain local replication cluster parameters
    local_replication_cluster = resource.Body('local_replication_cluster',
                                              type=ReplicationCluster)
    #: Active-active domain remote replication cluster parameters
    remote_replication_cluster = resource.Body(
        'remote_replication_cluster',
        type=ReplicationCluster)


class ActiveDomains(resource.Resource):
    """SDRS Active-Active Domain Resource"""
    base_path = '/active-domains'

    #: capabilities
    allow_list = True

    #: Properties
    #: List of available domains
    domains = resource.Body('domains', type=list, list_type=Domains)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import exceptions


class SignatureBind(resource.Resource):
    base_path = f'/apigw/instances/%(gateway_id)s/sign-bindings'

    allow_list = True
    allow_create = True
    allow_delete = True

    resources_key = 'bindings'

    _query_mapping = resource.QueryParameters(
        'limit', 'offset', 'api_id',
        'sign_id', 'sign_name', 'env_id'
    )

    # Properties
    gateway_id = resource.URI('gateway_id')
    # Signature key ID.
    sign_id = resource.Body('sign_id')
    # API publication record ID.
    publish_ids = resource.Body('publish_ids', type=list)

    # Attributes
    # API publication record ID.
    publish_id = resource.Body('publish_id', type=str)
    # API ID.
    api_id = resource.Body('api_id', type=str)
    # Name of the API group to which the API belongs.
    group_name = resource.Body('group_name', type=str)
    # Binding time.
    bind_at = resource.Body('binding_time', type=str)
    # ID of the environment in which the API has been published.
    env_id = resource.Body('env_id', type=str)
    # Name of the environment in which the API has been published.
    env_name = resource.Body('env_name', type=str)
    # API type.
    api_type = resource.Body('api_type', type=str)
    # API name.
    api_name = resource.Body('api_name', type=str)
    # API description.
    api_remark = resource.Body('api_remark', type=str)
    # Signature key name.
    sign_name = resource.Body('sign_name', type=str)
    # Request method.
    # Enumeration values:
    # GET
    # POST
    # DELETE
    # PUT
    # PATCH
    # HEAD
    # OPTIONS
    # ANY
    req_method = resource.Body('req_method', type=str)
    # Signature key.
    sign_key = resource.Body('sign_key', type=str)
    # Signature secret.
    sign_secret = resource.Body('sign_secret', type=str)
    # Signature key type.
    sign_type = resource.Body('sign_type', type=str)

    bindings = resource.Body('bindings', type=list)

    def create(
            self,
            session,
            prepend_key=True,
            base_path=None,
            *,
            resource_request_key=None,
            resource_response_key=None,
            microversion=None,
            **params,
    ):
        """Create a remote resource based on this instance.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param prepend_key: A boolean indicating whether the resource_key
            should be prepended in a resource creation request. Default to
            True.
        :param str base_path: Base part of the URI for creating resources, if
            different from :data:`~openstack.resource.Resource.base_path`.
        :param str resource_request_key: Overrides the usage of
            self.resource_key when prepending a key to the request body.
            Ignored if `prepend_key` is false.
        :param str resource_response_key: Overrides the usage of
            self.resource_key when processing response bodies.
            Ignored if `prepend_key` is false.
        :param str microversion: API version to override the negotiated one.
        :param dict params: Additional params to pass.
        :return: This :class:`Resource` instance.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
            :data:`Resource.allow_create` is not set to ``True``.
        """
        if not self.allow_create:
            raise exceptions.MethodNotSupported(self, 'create')

        session = self._get_session(session)
        if microversion is None:
            microversion = self._get_microversion(session)
        requires_id = (
            self.create_requires_id
            if self.create_requires_id is not None
            else self.create_method == 'PUT'
        )

        # Construct request arguments.
        request_kwargs = {
            "requires_id": requires_id,
            "prepend_key": prepend_key,
            "base_path": base_path,
        }
        if resource_request_key is not None:
            request_kwargs['resource_request_key'] = resource_request_key

        if self.create_exclude_id_from_body:
            self._body._dirty.discard("id")

        if self.create_method == 'PUT':
            request = self._prepare_request(**request_kwargs)
            response = session.put(
                request.url,
                json=request.body,
                headers=request.headers,
                microversion=microversion,
                params=params,
            )
        elif self.create_method == 'POST':
            request = self._prepare_request(**request_kwargs)
            response = session.post(
                request.url,
                json=request.body,
                headers=request.headers,
                microversion=microversion,
                params=params,
            )
        else:
            raise exceptions.ResourceFailure(
                "Invalid create method: %s" % self.create_method
            )

        has_body = (
            self.has_body
            if self.create_returns_body is None
            else self.create_returns_body
        )
        self.microversion = microversion

        response_kwargs = {
            "has_body": has_body,
        }
        if resource_response_key is not None:
            response_kwargs['resource_response_key'] = resource_response_key

        if has_body is None:
            has_body = self.has_body
        exceptions.raise_from_response(response)
        bindings = []
        if has_body and response.status_code == 201:
            if hasattr(self, 'bindings'):
                body = response.json()
                bindings = [
                    SignatureBind.existing(**data)
                    for data in body['bindings']
                ]
        self.bindings = bindings
        # direct comparision to False since we need to rule out None
        if self.has_body and self.create_returns_body is False:
            # fetch the body if it's required but not returned by create
            fetch_kwargs = {}
            if resource_response_key is not None:
                fetch_kwargs = {'resource_response_key': resource_response_key}
            return self.fetch(session, **fetch_kwargs)
        return self


class NotBoundApi(resource.Resource):
    base_path = f'/apigw/instances/%(gateway_id)s/sign-bindings/unbinded-apis'

    allow_list = True

    resources_key = 'apis'

    _query_mapping = resource.QueryParameters(
        'limit', 'offset', 'sign_id',
        'env_id', 'api_id', 'api_name', 'group_id'
    )

    # Properties
    gateway_id = resource.URI('gateway_id')

    # Attributes
    # API authentication mode.
    auth_type = resource.Body('auth_type', type=str)
    # Name of the environment in which the API has been published.
    run_env_name = resource.Body('run_env_name', type=str)
    # Name of the API group to which the API belongs.
    group_name = resource.Body('group_name', type=str)
    # Publication record ID.
    publish_id = resource.Body('publish_id', type=str)
    # ID of the API group to which the API belongs.
    group_id = resource.Body('group_id', type=str)
    # API name.
    name = resource.Body('name', type=str)
    # API description.
    remark = resource.Body('remark', type=str)
    # ID of the environment in which the API has been published.
    run_env_id = resource.Body('run_env_id', type=str)
    # API request address.
    req_uri = resource.Body('req_uri', type=str)
    # API type.
    type = resource.Body('type', type=int)
    # Name of the signature key bound to the API.
    signature_name = resource.Body('signature_name', type=str)
    # Request method.
    # Enumeration values:
    # GET
    # POST
    # DELETE
    # PUT
    # PATCH
    # HEAD
    # OPTIONS
    # ANY
    req_method = resource.Body('req_method', type=str)


class BoundApi(resource.Resource):
    base_path = f'/apigw/instances/%(gateway_id)s/sign-bindings/binded-apis'

    allow_list = True

    resources_key = 'bindings'

    _query_mapping = resource.QueryParameters(
        'limit', 'offset', 'sign_id',
        'env_id', 'api_id', 'api_name', 'group_id'
    )

    # Properties
    gateway_id = resource.URI('gateway_id')

    # Attributes
    # API publication record ID.
    publish_id = resource.Body('publish_id', type=str)
    # API ID.
    api_id = resource.Body('api_id', type=str)
    # Name of the API group to which the API belongs.
    group_name = resource.Body('group_name', type=str)
    # Binding time.
    bind_at = resource.Body('binding_time', type=str)
    # ID of the environment in which the API has been published.
    env_id = resource.Body('env_id', type=str)
    # Name of the environment in which the API has been published.
    env_name = resource.Body('env_name', type=str)
    # API type.
    api_type = resource.Body('api_type', type=str)
    # API name.
    api_name = resource.Body('api_name', type=str)
    # API description.
    api_remark = resource.Body('api_remark', type=str)
    # Signature key name.
    sign_name = resource.Body('sign_name', type=str)
    # Request method.
    # Enumeration values:
    # GET
    # POST
    # DELETE
    # PUT
    # PATCH
    # HEAD
    # OPTIONS
    # ANY
    req_method = resource.Body('req_method', type=str)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource
from openstack import utils


# class MetaDataSpec(resource.Resource):

# Properties
# Number of returned results / alarms
# count = resource.Body('count')
# Indicates pagination marker
# marker = resource.Body('marker')
# Number of total queried results / alarms
# total = resource.Body('total')


class AlarmActionsSpec(resource.Resource):

    # Properties
    # notification list ID
    notificationList = resource.Body('notificationList')
    # Indicates the type of action triggered by an alarm.
    # Value can be notication or autoscaling
    type = resource.Body('type')


class OkActionsSpec(resource.Resource):

    # Properties
    # notification list ID
    notificationList = resource.Body('notificationList')
    # Indicates the type of action triggered by an alarm.
    # Value can be notication or autoscaling
    type = resource.Body('type')


class ConditionSpec(resource.Resource):

    # Properties
    # indicates the comparison operator
    # values can be <,=,>,>= or <=
    comparison_operator = resource.Body('comparison_operator')
    # Indicates how many consecutive times an alarm has
    # been generated
    count = resource.Body('count', type=int)
    # indicates the data rollup method
    # values: max, min, average, sum, variance
    filter = resource.Body('filter')
    # Indicates the interval (in seconds) for checking
    # whether the configured alarm rules are met
    period = resource.Body('period', type=int)
    # Data unit
    unit = resource.Body('unit')
    # Alarm threshold
    value = resource.Body('value', type=int)


class DimensionsSpec(resource.Resource):

    # Properties
    #: dimension.name: object type e.g. ECS (instance_id)
    name = resource.Body('name')
    #: dimension.value: object id e.g. ECS ID
    value = resource.Body('value')


class MetricSpec(resource.Resource):

    # Properties
    # List of metric dimensions
    dimensions = resource.Body('dimensions', type=list,
                               list_type=DimensionsSpec)
    # Specifies the metric name
    metric_name = resource.Body('metric_name')
    # Metric Namespace
    namespace = resource.Body('namespace')


class Alarm(resource.Resource):

    resources_key = 'metric_alarms'
    base_path = '/alarms'

    # capabilities
    allow_commit = True
    allow_create = True
    allow_fetch = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'limit', 'order', 'start'
    )

    # Properties
    # Specifies the action triggered by an alarm.
    alarm_actions = resource.Body('alarm_actions', type=AlarmActionsSpec)
    # Indicates whether an action will be triggered by an alarm
    # True: action will be triggered
    # False: action will not be triggered
    alarm_action_enabled = resource.Body('alarm_action_enabled', type=bool)
    # Description of the alarm
    alarm_description = resource.Body('alarm_description')
    # Alarm is enabled (True) or disabled (False)
    alarm_enabled = resource.Body('alarm_enabled', type=bool)
    # alarm rule ID
    alarm_id = resource.Body('alarm_id', alternate_id=True)
    # alarm severity
    # values: 1: critical, 2: major, 3: minor, 4: informational alarm
    alarm_level = resource.Body('alarm_level', type=int)
    # Alarm status
    # ok: alarm status is normal
    # alarm: an alarm is generated
    # insufficient_data: required data is insufficient
    alarm_state = resource.Body('alarm_state')
    # Name of the alarm
    name = resource.Body('alarm_name')
    # Indicates the action triggered by clearing an alarm
    ok_actions = resource.Body('ok_actions', type=OkActionsSpec)
    # Describes alarm triggering condititon
    condition = resource.Body('condition', type=ConditionSpec)
    # Specification of specific alarm
    metric = resource.Body('metric', type=MetricSpec)
    # Time when alarm status changed
    # UNIX timestamp in ms
    update_time = resource.Body('update_time')

    def _translate_response(self, response, has_body=None, error_message=None,
                            resource_response_key=None):
        """Given a KSA response, inflate this instance with its data

        DELETE operations don't return a body, so only try to work
        with a body when has_body is True.

        This method updates attributes that correspond to headers
        and body on this instance and clears the dirty set.
        """
        if has_body is None:
            has_body = self.has_body
        exceptions.raise_from_response(response, error_message=error_message)
        if has_body:
            try:
                body = response.json()
                if self.resources_key and self.resources_key in body:
                    body = body[self.resources_key][0]
                body_attrs = self._consume_body_attrs(body)
                self._body.attributes.update(body_attrs)
                self._body.clean()

            except ValueError:
                # Server returned not parse-able response (202, 204, etc)
                # Do simply nothing
                pass

        headers = self._consume_header_attrs(response.headers)
        self._header.attributes.update(headers)
        self._header.clean()
        self._update_location()
        dict.update(self, self.to_dict())

    def _action(self, session, body):
        """Perform actions given the message body.

        """
        url = utils.urljoin(self.base_path, self.id, "action")
        response = session.put(
            url,
            json=body)
        exceptions.raise_from_response(response)
        return response

    def change_alarm_status(self, session):
        body = {
            "alarm_enabled": True
        }
        current_status = self.get('alarm_enabled')
        if current_status is True:
            body.update({'alarm_enabled': False})
        self._action(session, body)

    @classmethod
    def find(cls, session, name_or_id, ignore_missing=True, **params):
        """Find a resource by its name or id.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param name_or_id: This resource's identifier, if needed by
                           the request. The default is ``None``.
        :param bool ignore_missing: When set to ``False``
                    :class:`~openstack.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :param dict params: Any additional parameters to be passed into
                            underlying methods, such as to
                            :meth:`~openstack.resource.Resource.existing`
                            in order to pass on URI parameters.

        :return: The :class:`Resource` object matching the given name or id
                 or None if nothing matches.
        :raises: :class:`openstack.exceptions.DuplicateResource` if more
                 than one resource is found for this request.
        :raises: :class:`openstack.exceptions.ResourceNotFound` if nothing
                 is found and ignore_missing is ``False``.
        """
        session = cls._get_session(session)
        # Try to short-circuit by looking directly for a matching ID.
        try:
            match = cls.existing(
                id=name_or_id,
                connection=session._get_connection(),
                **params)
            return match.fetch(session, **params)
        except exceptions.SDKException:
            pass

        data = cls.list(session, **params)

        result = cls._get_one_match(name_or_id, data)
        if result is not None:
            return result

        if ignore_missing:
            return None
        raise exceptions.ResourceNotFound(
            "No %s found for %s" % (cls.__name__, name_or_id))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource
from openstack import utils

from otcextensions.sdk.rds.v3 import _base


class Configuration(_base.Resource):

    base_path = '/configurations'
    resources_key = 'configurations'
    resource_key = 'configuration'

    # capabilities
    allow_create = True
    allow_delete = True
    allow_commit = True
    allow_fetch = True
    allow_list = True

    #: Id of the configuration group
    configuration_id = resource.Body('configuration_id')
    configuration_name = resource.Body('configuration_name')
    #: *Type:str*
    id = resource.Body('id', alias='configuration_id')
    #: Name of the configuration group
    #: *Type:str*
    name = resource.Body('name', alias='configuration_name')
    #: Data store information
    #: *Type: dict*
    datastore = resource.Body('datastore', type=dict)
    #: Description of the configuration group
    #: *Type:str*
    description = resource.Body('description')
    #: Name of Datastore version
    #: *Type:str*
    datastore_version_name = resource.Body('datastore_version_name')
    #: Name of Datastore
    #: *Type:str*
    datastore_name = resource.Body('datastore_name')
    #: Individual Configuration values
    #: *Type:dict*
    configuration_parameters = resource.Body('configuration_parameters',
                                             type=list)
    #: Date of created
    #: *Type:str*
    created_at = resource.Body('created')
    #: Date of updated
    #: *Type:str*
    updated_at = resource.Body('updated')
    #: Indicates whether the parameter group is created by users.
    #: *Type:bool*
    is_user_defined = resource.Body('user_defined', type=bool)
    #: parameter group values defined by users
    #: *Type: dict*
    values = resource.Body('values', type=dict)

    #: Results of the configuration apply per instance
    #: *Type:list*
    apply_results = resource.Body('apply_results', type=list)

    @classmethod
    def find(cls, session, name_or_id, ignore_missing=True, **params):
        instance = super(Configuration, cls).find(
            session, name_or_id,
            ignore_missing=ignore_missing,
            **params)
        if instance:
            return instance.fetch(session)
        return

    def create(self, session, prepend_key=False, base_path=None):
        return super(Configuration, self).create(session,
                                                 prepend_key=prepend_key,
                                                 base_path=base_path)

    def commit(self, session, prepend_key=False, **further_attrs):
        return super(Configuration, self).commit(
            session,
            prepend_key=prepend_key,
            **further_attrs)

    def apply(self, session, instances):
        """Apply configuration to the given instances
        """
        url = utils.urljoin(self.base_path, self.id, 'apply')
        body = {'instance_ids': instances}
        response = session.put(
            url,
            json=body)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
'''
Update MRS Job
'''
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')

attrs = {
    "name": "my-mapreduce-job-update",
    "mains": [],
    "libs": [
        "2628d0e4-6109-4a09-a338-c4ee1b0963ed"
    ],
    "is_protected": False,
    "interface": [],
    "is_public": False,
    "type": "MapReduce",
    "description": "This is the Map Reduce job template"
}

job = 'job1'
job = conn.mrs.find_job(name_or_id=job)
job = conn.mrs.update_job(job=job, **attrs)
print(job)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""CSS ELK cluster v1 action implementations"""

import logging

from osc_lib import exceptions
from osc_lib import utils
from osc_lib.cli import format_columns
from osc_lib.cli import parseractions
from osc_lib.command import command

from otcextensions.common import cli_utils
from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


DISK_TYPE_CHOICES = ['common', 'high', 'ultrahigh']
NODE_TYPE_CHOICES = ['ess', 'ess-cold', 'ess-client', 'ess-master']
VOLUME_TYPE_CHOICES = ['ULTRAHIGH', 'COMMON', 'HIGH']
UPGRADE_TYPE_CHOICES = ['same', 'cross', 'cross-engine']


_formatters = {
    'nodes': cli_utils.YamlFormat,
    'elb_whitelist': cli_utils.YamlFormat,
    'datastore': cli_utils.YamlFormat,
    'tags': cli_utils.YamlFormat,
    'action_progress': format_columns.DictColumn,
    'actions': format_columns.ListColumn,
    'endpoints': cli_utils.YamlFormat,
}


def set_attributes_for_print(obj):
    for data in obj:
        if getattr(data, 'datastore'):
            setattr(data, 'type', data.datastore['type'])
            setattr(data, 'version', data.datastore['version'])
        yield data


def _get_columns(item):
    column_map = {}
    hidden = [
        'location',
    ]
    return sdk_utils.get_osc_show_columns_for_sdk_resource(
        item, column_map, hidden
    )


def translate_response(func):
    def new(self, *args, **kwargs):
        obj = func(self, *args, **kwargs)
        setattr(obj, 'num_nodes', len(obj.nodes))
        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)
        return (display_columns, data)

    new.__name__ = func.__name__
    new.__doc__ = func.__doc__
    return new


class CreateCluster(command.ShowOne):
    _description = _('Create a new CSS cluster instance.')

    def get_parser(self, prog_name):
        parser = super(CreateCluster, self).get_parser(prog_name)
        parser.add_argument('name', metavar='<name>', help=_('Cluster Name.'))
        parser.add_argument(
            '--datastore-type',
            metavar='{elasticsearch, opensearch}',
            choices=['elasticsearch', 'opensearch'],
            type=lambda s: s.lower(),
            default='elasticsearch',
            help=_(
                'Cluster type. Values:\n'
                '- elasticsearch\n'
                '- opensearch\n'
                'The default value is elasticsearch.'
            ),
        )
        parser.add_argument(
            '--datastore-version',
            metavar='<datastore_version>',
            default='7.10.2',
            help=_(
                'CSS Cluster Engine Versions.\n'
                'If datastore_type is `elasticsearch` supported versions: '
                '(7.6.2, 7.9.3, 7.10.2)\n'
                'If datastore_type is `opensearch` supported versions: '
                '(1.3.6)\n'
                '(default datastore_version: 7.10.2).'
            ),
        )
        parser.add_argument(
            '--availability-zone',
            metavar='<availability_zone>',
            help=_(
                'Separate multiple AZs with commas (,), for example, '
                'az1,az2. AZs must be unique. The number of nodes must be '
                'greater than or equal to the number of AZs.'
            ),
        )
        parser.add_argument(
            '--flavor',
            metavar='<flavor>',
            required=True,
            help=_('Cluster Instance flavor.'),
        )
        parser.add_argument(
            '--num-nodes',
            metavar='<num_nodes>',
            type=int,
            default=1,
            help=_(
                'Number of clusters nodes. The value range is 1 to 32. '
                '(default value: 1)'
            ),
        )
        disk_group = parser.add_argument_group('Volume Parameters')
        disk_group.add_argument(
            '--volume-size',
            metavar='<volume_size>',
            default=40,
            type=int,
            help=_(
                'Size of the instance disk volume in GB. '
                '(default value: 40)'
            ),
        )
        disk_group.add_argument(
            '--volume-type',
            metavar='{' + ','.join(DISK_TYPE_CHOICES) + '}',
            type=lambda s: s.upper(),
            default='COMMON',
            dest='volume_type',
            choices=[s.upper() for s in DISK_TYPE_CHOICES],
            help=_(
                'Volume type. Supported types: COMMON, HIGH, ULTRAHIGH. '
                '(default value: COMMON)'
            ),
        )
        network_group = parser.add_argument_group('Network Parameters')
        network_group.add_argument(
            '--router-id',
            metavar='<router_id>',
            required=True,
            help=_('Router ID.'),
        )
        network_group.add_argument(
            '--network-id',
            metavar='<network_id>',
            required=True,
            help=_('Network ID.'),
        )
        network_group.add_argument(
            '--security-group-id',
            metavar='<security_group_id>',
            required=True,
            help=_('Security group ID.'),
        )
        parser.add_argument(
            '--https-enable',
            action='store_true',
            help=_('Whether communication is encrypted on the cluster.'),
        )
        parser.add_argument(
            '--cmk-id',
            metavar='<cmk_id>',
            help=_(
                'Encryption Key Id. '
                'The system encryption is used or cluster encryption.'
                'The Default Master Keys cannot be used to create grants.'
            ),
        )
        parser.add_argument(
            '--admin-pwd',
            metavar='<admin_pwd>',
            help=_('Password of the cluster user admin in security mode.'),
        )
        parser.add_argument(
            '--backup-policy',
            metavar='period=<period>,prefix=<prefix>,keepday=<keepday>',
            required_keys=['period', 'prefix', 'keepday'],
            optional_keys=['bucket', 'agency', 'basepath'],
            dest='backup_policy',
            action=parseractions.MultiKeyValueAction,
            help=_(
                'Automatic backup creation policy. '
                'This function is enabled by default.\n'
                'The following keys are required:\n'
                'period=<period>: Time when a snapshot is created '
                'every day.\n'
                'prefix=<prefix>: Prefix of the name of the snapshot '
                'that is automatically created.\n'
                'keepday=<keepday>: Number of days for which automatically '
                'created snapshots are reserved. Value range: 1 to 90.\n'
                'Optional Keys:\n'
                'bucket=<bucket>: OBS bucket used for storing backup.\n'
                'basepath=<basepath>: Storage path of the snapshot in '
                'the OBS bucket.\n'
                'agency=<agency>: IAM agency used to access OBS.'
            ),
        )
        parser.add_argument(
            '--tag',
            action=parseractions.MultiKeyValueAction,
            metavar='key=<key>,value=<value>',
            required_keys=['key', 'value'],
            dest='tags',
            help=_(
                'key=<key>: Tag key. The value can contain 1 to 36 '
                'characters. Only digits, letters, hyphens (-) and '
                'underscores (_) are allowed.\n'
                'value=<value>: Tag value. The value can contain 0 to 43 '
                'characters. Only digits, letters, hyphens (-) and '
                'underscores (_) are allowed.'
            ),
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help=('Wait for Cluster.'),
        )
        parser.add_argument(
            '--timeout',
            metavar='<timeout>',
            type=int,
            default=1200,
            help=_('Timeout for the wait in seconds (default 1200 seconds).'),
        )
        return parser

    @translate_response
    def take_action(self, parsed_args):
        client = self.app.client_manager.css

        attrs = {
            'name': parsed_args.name,
            'instanceNum': parsed_args.num_nodes,
            'datastore': {
                'version': parsed_args.datastore_version,
                'type': parsed_args.datastore_type,
            },
            'instance': {
                'flavorRef': parsed_args.flavor,
                'volume': {
                    'volume_type': parsed_args.volume_type,
                    'size': parsed_args.volume_size,
                },
                'nics': {
                    'vpcId': parsed_args.router_id,
                    'netId': parsed_args.network_id,
                    'securityGroupId': parsed_args.security_group_id,
                },
            },
        }

        availability_zone = parsed_args.availability_zone
        if availability_zone:
            attrs['instance']['availability_zone'] = availability_zone

        if parsed_args.https_enable:
            attrs['httpsEnable'] = True
            attrs['authorityEnable'] = True
            admin_password = parsed_args.admin_pwd
            if admin_password:
                attrs['adminPwd'] = admin_password
            else:
                raise exceptions.CommandError(
                    'Following arguments is required: --admin-pwd '
                    '(admin_pwd is mandatary in https_enable mode.)'
                )
        elif parsed_args.admin_pwd and not parsed_args.https_enable:
            attrs['authorityEnable'] = True
            attrs['adminPwd'] = parsed_args.admin_pwd

        if parsed_args.cmk_id:
            attrs['diskEncryption'] = {
                'systemEncrypted': 1,
                'systemCmkid': parsed_args.cmk_id,
            }
        backup_policy = parsed_args.backup_policy
        if backup_policy:
            if len(backup_policy) > 1:
                msg = '--backup-policy option cannot be repeated'
                raise exceptions.CommandError(msg)
            else:
                backup_policy = backup_policy[0]
                backup_policy['keepday'] = int(backup_policy['keepday'])
                if backup_policy.get('basepath'):
                    backup_policy['basePath'] = backup_policy['basepath']
                    del backup_policy['basepath']
                attrs['backupStrategy'] = backup_policy
        if parsed_args.tags:
            attrs['tags'] = parsed_args.tags

        cluster = client.create_cluster(**attrs)
        if parsed_args.wait:
            client.wait_for_cluster(
                cluster.id, parsed_args.timeout, print_status=True
            )
        return client.get_cluster(cluster.id)


class ListClusters(command.Lister):
    _description = _('List CSS Clusters.')
    columns = (
        'ID',
        'Name',
        'Type',
        'Version',
        'Status',
        'Created At',
    )

    def get_parser(self, prog_name):
        parser = super(ListClusters, self).get_parser(prog_name)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css
        data = client.clusters()

        return (
            self.columns,
            (
                utils.get_item_properties(
                    s, self.columns, formatters=_formatters
                )
                for s in set_attributes_for_print(data)
            ),
        )


class ListClusterNodes(command.Lister):
    _description = _('List CSS Cluster Nodes.')
    columns = (
        'ID',
        'Name',
        'IP',
        'Type',
        'Volume',
        'Availability Zone',
        'Status',
    )

    def get_parser(self, prog_name):
        parser = super(ListClusterNodes, self).get_parser(prog_name)
        parser.add_argument(
            'cluster', metavar='<cluster>', help=_('Cluster name or ID.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css

        cluster = client.find_cluster(
            parsed_args.cluster, ignore_missing=False
        )

        _formatters = {'Volume': cli_utils.YamlFormat}
        return (
            self.columns,
            (
                utils.get_item_properties(
                    node, self.columns, formatters=_formatters
                )
                for node in cluster.nodes
            ),
        )


class ShowCluster(command.ShowOne):
    _description = _('Show details of a CSS cluster')

    def get_parser(self, prog_name):
        parser = super(ShowCluster, self).get_parser(prog_name)
        parser.add_argument(
            'cluster', metavar='<cluster>', help=_('Cluster name or ID.')
        )
        return parser

    @translate_response
    def take_action(self, parsed_args):
        client = self.app.client_manager.css

        cluster = client.find_cluster(
            parsed_args.cluster, ignore_missing=False
        )

        if not getattr(cluster, 'nodes'):
            cluster = client.get_cluster(cluster)

        return cluster


class RestartCluster(command.Command):
    _description = _('Restart a CSS cluster')

    def get_parser(self, prog_name):
        parser = super(RestartCluster, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID or Name of the CSS cluster to be restart.'),
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help=('Wait for Cluster to Restart.'),
        )
        parser.add_argument(
            '--timeout',
            metavar='<timeout>',
            type=int,
            default=300,
            help=_('Timeout for the wait in seconds (default 300 seconds).'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css
        cluster = client.find_cluster(
            parsed_args.cluster, ignore_missing=False
        )
        client.restart_cluster(cluster)
        if parsed_args.wait:
            client.wait_for_cluster(cluster.id, parsed_args.timeout)


class ExtendClusterNodes(command.Command):
    _description = _('Scaling Out a Cluster with Special Nodes.')

    def get_parser(self, prog_name):
        parser = super(ExtendClusterNodes, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID or Name of the CSS cluster to be extended.'),
        )
        parser.add_argument(
            '--extend',
            metavar='type=<type>,nodesize=<nodesize>,disksize=<disksize>',
            required_keys=['type', 'nodesize', 'disksize'],
            required=True,
            action=parseractions.MultiKeyValueAction,
            help=_(
                'Extend Cluster Nodes.'
                'Type: ess, ess-cold, ess-master, and ess-client.\n'
                'For type: ess-master and ess-client disksize cannot '
                'be extended.\n'
                'Examples:\n'
                '--extend type=ess,disksize=60,nodesize=2 '
                ' --extend type=ess-master,disksize=0,nodesize=2'
            ),
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help=('Wait for Cluster Scaling Task to complete.'),
        )
        parser.add_argument(
            '--timeout',
            metavar='<timeout>',
            type=int,
            default=1200,
            help=_('Timeout for the wait in seconds (default 1200 seconds).'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css
        cluster = client.find_cluster(
            parsed_args.cluster, ignore_missing=False
        )
        attrs = {'grow': parsed_args.extend}
        client.extend_cluster_nodes(cluster, **attrs)
        if parsed_args.wait:
            client.wait_for_cluster(cluster.id, parsed_args.timeout)


class ExtendCluster(command.Command):
    _description = _("Scaling Out a Cluster's with only Common Nodes.")

    def get_parser(self, prog_name):
        parser = super(ExtendCluster, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID or Name of the CSS cluster to be extended.'),
        )
        parser.add_argument(
            '--add-nodes',
            metavar='<add_nodes>',
            type=int,
            required=True,
            help=_('Number of css nodes to be scaled out.'),
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help=('Wait for Cluster Scaling Task to complete.'),
        )
        parser.add_argument(
            '--timeout',
            metavar='<timeout>',
            type=int,
            default=1200,
            help=_('Timeout for the wait in seconds (default 1200 seconds).'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css
        cluster = client.find_cluster(
            parsed_args.cluster, ignore_missing=False
        )
        client.extend_cluster(cluster, parsed_args.add_nodes)
        if parsed_args.wait:
            client.wait_for_cluster(cluster.id, parsed_args.timeout)


class DeleteCluster(command.Command):
    _description = _('Delete CSS Cluster(s)')

    def get_parser(self, prog_name):
        parser = super(DeleteCluster, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            nargs='+',
            help=_('ID or Name of the CSS cluster(s) to be deleted.'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css
        result = 0
        for name_or_id in parsed_args.cluster:
            try:
                cluster = client.find_cluster(name_or_id, ignore_missing=False)
                client.delete_cluster(cluster, ignore_missing=False)
            except Exception as e:
                result += 1
                LOG.error(
                    _(
                        'Failed to delete cluster(s) with '
                        "ID or Name '%(cluster)s': %(e)s"
                    ),
                    {'cluster': name_or_id, 'e': e},
                )
        if result > 0:
            total = len(parsed_args.cluster)
            msg = _(
                '%(result)s of %(total)s Cluster(s) failed ' 'to delete.'
            ) % {'result': result, 'total': total}
            raise exceptions.CommandError(msg)


class UpdateClusterName(command.ShowOne):
    _description = _('Change the name of a cluster.')

    def get_parser(self, prog_name):
        parser = super(UpdateClusterName, self).get_parser(prog_name)
        parser.add_argument(
            'cluster', metavar='<cluster>', help=_('Cluster name or ID.')
        )
        parser.add_argument(
            '--new-name',
            required=True,
            metavar='<new_name>',
            help=_('New cluster name.'),
        )
        return parser

    @translate_response
    def take_action(self, parsed_args):
        client = self.app.client_manager.css

        cluster = client.find_cluster(
            parsed_args.cluster, ignore_missing=False
        )

        client.update_cluster_name(cluster, parsed_args.new_name)

        return client.get_cluster(cluster)


class UpdateClusterPassword(command.Command):
    _description = _('Change the password of a cluster.')

    def get_parser(self, prog_name):
        parser = super(UpdateClusterPassword, self).get_parser(prog_name)
        parser.add_argument(
            'cluster', metavar='<cluster>', help=_('Cluster name or ID.')
        )
        parser.add_argument(
            '--new-password',
            required=True,
            metavar='<new_password>',
            help=_('New password.'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css

        cluster = client.find_cluster(
            parsed_args.cluster, ignore_missing=False
        )

        client.update_cluster_password(cluster, parsed_args.new_password)


class UpdateClusterSecurityGroup(command.Command):
    _description = _('Change the security group after a cluster is created')

    def get_parser(self, prog_name):
        parser = super(UpdateClusterSecurityGroup, self).get_parser(prog_name)
        parser.add_argument(
            'cluster', metavar='<cluster>', help=_('Cluster name or ID.')
        )
        parser.add_argument(
            '--security-group',
            required=True,
            metavar='<security_group>',
            help=_('New security group id or name.'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css
        network_client = self.app.client_manager.network

        cluster = client.find_cluster(
            parsed_args.cluster, ignore_missing=False
        )
        security_group = network_client.find_security_group(
            parsed_args.security_group, ignore_missing=False
        )

        client.update_cluster_security_group(cluster, security_group.id)


class UpdateClusterSecurityMode(command.Command):
    _description = _('Change the security mode of a cluster.')

    def get_parser(self, prog_name):
        parser = super(UpdateClusterSecurityMode, self).get_parser(prog_name)
        parser.add_argument(
            'cluster', metavar='<cluster>', help=_('Cluster name or ID.')
        )
        parser.add_argument(
            '--authority-enable',
            action='store_true',
            help=('Indicates whether to enable the security mode.'),
        )
        parser.add_argument(
            '--admin-pwd',
            metavar='<admin_pwd>',
            help=_('Cluster password in security mode.'),
        )
        parser.add_argument(
            '--https-enable',
            action='store_true',
            help=('Indicates whether to enable HTTPS.'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css
        attrs = {}

        if parsed_args.authority_enable:
            attrs['authority_enable'] = True

        if parsed_args.https_enable:
            attrs['https_enable'] = True

        if parsed_args.admin_pwd:
            attrs['admin_pwd'] = parsed_args.admin_pwd

        cluster = client.find_cluster(
            parsed_args.cluster, ignore_missing=False
        )

        client.update_cluster_security_mode(cluster, **attrs)


class UpdateClusterFlavor(command.Command):
    _description = _('Modify the specifications of a cluster.')

    def get_parser(self, prog_name):
        parser = super(UpdateClusterFlavor, self).get_parser(prog_name)
        parser.add_argument(
            'cluster', metavar='<cluster>', help=_('Cluster name or ID.')
        )
        parser.add_argument(
            '--node-type',
            metavar='{' + ','.join(NODE_TYPE_CHOICES) + '}',
            choices=NODE_TYPE_CHOICES,
            type=lambda s: s.lower(),
            help=('Type of the node to modify.'),
        )
        parser.add_argument(
            '--flavor',
            required=True,
            metavar='<flavor>',
            help=('ID of the new flavor.'),
        )
        parser.add_argument(
            '--check-replica',
            action='store_true',
            help=('Indicates whether to verify replicas.'),
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help=('Wait for Cluster.'),
        )
        parser.add_argument(
            '--timeout',
            metavar='<timeout>',
            type=int,
            default=1200,
            help=_('Timeout for the wait in seconds (default 1200 seconds).'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css
        attrs = {}

        if parsed_args.node_type:
            attrs['node_type'] = parsed_args.node_type

        attrs['new_flavor'] = parsed_args.flavor

        if parsed_args.check_replica:
            attrs['check_replica'] = True

        cluster = client.find_cluster(
            parsed_args.cluster, ignore_missing=False
        )

        client.update_cluster_flavor(cluster, **attrs)

        if parsed_args.wait:
            client.wait_for_cluster(cluster.id, parsed_args.timeout)


class ScaleInCluster(command.Command):
    _description = _('Scale in a cluster by removing specified nodes.')

    def get_parser(self, prog_name):
        parser = super(ScaleInCluster, self).get_parser(prog_name)
        parser.add_argument(
            'cluster', metavar='<cluster>', help=_('Cluster name or ID.')
        )
        parser.add_argument(
            '--nodes',
            nargs='+',
            required=True,
            metavar='<nodes>',
            help=_('IDs of the nodes to remove.'),
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help=('Wait for Cluster.'),
        )
        parser.add_argument(
            '--timeout',
            metavar='<timeout>',
            type=int,
            default=1200,
            help=_('Timeout for the wait in seconds (default 1200 seconds).'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css

        cluster = client.find_cluster(
            parsed_args.cluster, ignore_missing=False
        )

        client.scale_in_cluster(cluster, parsed_args.nodes)

        if parsed_args.wait:
            client.wait_for_cluster(cluster.id, parsed_args.timeout)


class ScaleInClusterByNodeType(command.Command):
    _description = _('Remove instances of specific types.')

    def get_parser(self, prog_name):
        parser = super(ScaleInClusterByNodeType, self).get_parser(prog_name)
        parser.add_argument(
            'cluster', metavar='<cluster>', help=_('Cluster name or ID.')
        )
        parser.add_argument(
            '--ess',
            metavar='<reduce_num>',
            type=int,
            help=_('Reduce the ess nodes.'),
        )
        parser.add_argument(
            '--ess-master',
            metavar='<reduce_num>',
            dest='ess-master',
            type=int,
            help=_('Reduce the ess-master nodes.'),
        )
        parser.add_argument(
            '--ess-client',
            metavar='<reduce_num>',
            dest='ess-client',
            type=int,
            help=_('Reduce the ess-client nodes.'),
        )
        parser.add_argument(
            '--ess-cold',
            metavar='<reduce_num>',
            dest='ess-cold',
            type=int,
            help=_('Reduce the ess-cold nodes.'),
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help=('Wait for Cluster.'),
        )
        parser.add_argument(
            '--timeout',
            metavar='<timeout>',
            type=int,
            default=1200,
            help=_('Timeout for the wait in seconds (default 1200 seconds).'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css

        nodes = []

        for arg in ('ess', 'ess-master', 'ess-client', 'ess-cold'):
            value = getattr(parsed_args, arg)
            if value:
                nodes.append(
                    {
                        'type': arg,
                        'reducedNodeNum': value,
                    }
                )

        cluster = client.find_cluster(
            parsed_args.cluster, ignore_missing=False
        )

        client.scale_in_cluster_by_node_type(cluster, nodes)

        if parsed_args.wait:
            client.wait_for_cluster(cluster.id, parsed_args.timeout)


class UpdateClusterKernel(command.Command):
    _description = _('Upgrade cluster version.')

    def get_parser(self, prog_name):
        parser = super(UpdateClusterKernel, self).get_parser(prog_name)
        parser.add_argument(
            'cluster', metavar='<cluster>', help=_('Cluster name or ID.')
        )
        parser.add_argument(
            '--target-image-id',
            metavar='<target_image_id>',
            help=_('ID of the target image version.'),
        )
        parser.add_argument(
            '--upgrade-type',
            required=True,
            metavar='{' + ','.join(UPGRADE_TYPE_CHOICES) + '}',
            choices=UPGRADE_TYPE_CHOICES,
            type=lambda s: s.lower(),
            help=_('Upgrade type.'),
        )
        parser.add_argument(
            '--check-backup-indices',
            action='store_true',
            help=_('ID of the target image version.'),
        )
        parser.add_argument(
            '--agency',
            required=True,
            metavar='<agency>',
            help=_('Agency name.'),
        )
        parser.add_argument(
            '--check-cluster-load',
            action='store_true',
            help=_('Indicates whether to verify the load.'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css
        attrs = {}

        for arg in ('target_image_id', 'upgrade_type', 'agency'):
            value = getattr(parsed_args, arg)
            if value:
                attrs[arg] = value

        attrs['indices_backup_check'] = (
            True if parsed_args.check_backup_indices else False
        )

        if parsed_args.check_cluster_load:
            attrs['cluster_load_check'] = parsed_args.check_cluster_load

        cluster = client.find_cluster(
            parsed_args.cluster, ignore_missing=False
        )

        client.update_cluster_kernel(cluster, **attrs)


class ReplaceClusterNode(command.Command):
    _description = _('Replace a node in the cluster.')

    def get_parser(self, prog_name):
        parser = super(ReplaceClusterNode, self).get_parser(prog_name)
        parser.add_argument(
            'cluster', metavar='<cluster>', help=_('Cluster name or ID.')
        )
        parser.add_argument(
            '--node-id',
            required=True,
            metavar='<nodes_id>',
            help=_('IDs of the node to be replaced.'),
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help=('Wait for Cluster.'),
        )
        parser.add_argument(
            '--timeout',
            metavar='<timeout>',
            type=int,
            default=1200,
            help=_('Timeout for the wait in seconds (default 1200 seconds).'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css

        cluster = client.find_cluster(
            parsed_args.cluster, ignore_missing=False
        )

        client.replace_cluster_node(cluster, parsed_args.node_id)

        if parsed_args.wait:
            client.wait_for_cluster(cluster.id, parsed_args.timeout)


class AddClusterNodes(command.Command):
    _description = _('Add master and client nodes to a cluster.')

    def get_parser(self, prog_name):
        parser = super(AddClusterNodes, self).get_parser(prog_name)
        parser.add_argument(
            'cluster', metavar='<cluster>', help=_('Cluster name or ID.')
        )
        parser.add_argument(
            '--node-type',
            required=True,
            metavar='<node_type>',
            help=_('Node type.'),
        )
        parser.add_argument(
            '--flavor', required=True, metavar='<flavor>', help=_('Flavor ID.')
        )
        parser.add_argument(
            '--node-size',
            required=True,
            type=int,
            metavar='<node_size>',
            help=_('Number of nodes.'),
        )
        parser.add_argument(
            '--volume-type',
            required=True,
            metavar='{' + ','.join(VOLUME_TYPE_CHOICES) + '}',
            choices=VOLUME_TYPE_CHOICES,
            type=lambda s: s.upper(),
            help=_('Node storage type.'),
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help=('Wait for Cluster.'),
        )
        parser.add_argument(
            '--timeout',
            metavar='<timeout>',
            type=int,
            default=1200,
            help=_('Timeout for the wait in seconds (default 1200 seconds).'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css

        attrs = {
            'node_type': parsed_args.node_type,
            'flavor': parsed_args.flavor,
            'node_size': parsed_args.node_size,
            'volume_type': parsed_args.volume_type,
        }

        cluster = client.find_cluster(
            parsed_args.cluster, ignore_missing=False
        )

        client.add_cluster_nodes(cluster, **attrs)

        if parsed_args.wait:
            client.wait_for_cluster(cluster.id, parsed_args.timeout)


class RetryClusterUpgradeJob(command.Command):
    _description = _('Retry a task or terminate the impact of a task.')

    def get_parser(self, prog_name):
        parser = super(RetryClusterUpgradeJob, self).get_parser(prog_name)
        parser.add_argument(
            'cluster', metavar='<cluster>', help=_('Cluster name or ID.')
        )
        parser.add_argument(
            '--job-id',
            required=True,
            metavar='<job_id>',
            help=_('ID of the task to be retried.'),
        )
        parser.add_argument(
            '--retry-mode',
            metavar='<retry_mode>',
            default='abort',
            help=_(
                """If this parameter is not left blank,
                the impact of the task is terminated."""
            ),
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help=('Wait for Cluster.'),
        )
        parser.add_argument(
            '--timeout',
            metavar='<timeout>',
            type=int,
            default=1200,
            help=_('Timeout for the wait in seconds (default 1200 seconds).'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css
        attrs = {}

        attrs['job_id'] = parsed_args.job_id

        attrs['retry_mode'] = parsed_args.retry_mode

        cluster = client.find_cluster(
            parsed_args.cluster, ignore_missing=False
        )

        client.retry_cluster_upgrade_job(cluster, **attrs)

        if parsed_args.wait:
            client.wait_for_cluster(cluster.id, parsed_args.timeout)


class ListClusterVersionUpgrades(command.Lister):
    _description = _('List available upgradable versions.')

    column_headers = [
        'Datastore Type',
        'Datastore Version',
        'Image Name',
        'Id',
        'Description',
        'Priority',
    ]

    columns = [
        'datastore_type',
        'datastore_version',
        'display_name',
        'id',
        'image_desc',
        'priority',
    ]

    def get_parser(self, prog_name):
        parser = super(ListClusterVersionUpgrades, self).get_parser(prog_name)
        parser.add_argument(
            'cluster', metavar='<cluster>', help=_('Cluster name or ID.')
        )
        parser.add_argument(
            '--upgrade-type',
            required=True,
            metavar='{same, cross}',
            choices=['same', 'cross'],
            type=lambda s: s.lower(),
            help=_('Version type.'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css

        cluster = client.find_cluster(
            parsed_args.cluster, ignore_missing=False
        )

        obj = client.get_cluster_version_upgrades(
            cluster, parsed_args.upgrade_type
        )

        image_info_list = obj.image_info_list

        _formatters = {}

        return (
            self.column_headers,
            (
                utils.get_item_properties(
                    s, self.columns, formatters=_formatters
                )
                for s in image_info_list
            ),
        )


class ShowClusterUpgradeStatus(command.Lister):
    _description = _('List CSS Cluster Nodes.')
    columns = (
        'ID',
        'Image Info',
        'Execute Times',
        'Start Time',
        'End Time',
        'Status',
    )

    def get_parser(self, prog_name):
        parser = super(ShowClusterUpgradeStatus, self).get_parser(prog_name)
        parser.add_argument(
            'cluster', metavar='<cluster>', help=_('Cluster name or ID.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css

        cluster = client.find_cluster(
            parsed_args.cluster, ignore_missing=False
        )

        data = client.get_cluster_upgrade_status(cluster)

        _formatters = {'Image Info': cli_utils.YamlFormat}

        return (
            self.columns,
            (
                utils.get_item_properties(
                    s, self.columns, formatters=_formatters
                )
                for s in data
            ),
        )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''CCE Cluster v2 action implementations'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


CONTAINER_NET_MODE_CHOICES = ['overlay_l2', 'underlay_ipvlan', 'vpc-router']
CLUSTER_TYPES = ['VirtualMachine']


def _flatten_cluster(obj):
    data = {
        'id': obj.id,
        'name': obj.name,
        'status': obj.status.status,
        'type': obj.spec.type,
        'flavor': obj.spec.flavor,
        'router_id': obj.spec.host_network.router_id,
        'network_id': obj.spec.host_network.network_id,
        'version': obj.spec.version
    }

    endpoints = obj.status.get('endpoints')
    if endpoints:
        data['endpoint'] = endpoints.get('external_otc')

    return data


class ListCCECluster(command.Lister):
    _description = _('List CCE Clusters')
    columns = ('ID', 'name', 'status', 'flavor', 'version', 'endpoint')

    def get_parser(self, prog_name):
        parser = super(ListCCECluster, self).get_parser(prog_name)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cce

        data = client.clusters()

        table = (self.columns,
                 (utils.get_dict_properties(
                     _flatten_cluster(s), self.columns,
                 ) for s in data))
        return table


class ShowCCECluster(command.ShowOne):
    _description = _('Show single Cluster details')
    columns = ('ID', 'name', 'type', 'status', 'version', 'endpoint',
               'flavor', 'router_id', 'network_id')

    def get_parser(self, prog_name):
        parser = super(ShowCCECluster, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID of the cluster')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cce

        obj = client.find_cluster(parsed_args.cluster, ignore_missing=False)

        # display_columns, columns = _get_columns(obj)
        data = utils.get_dict_properties(_flatten_cluster(obj), self.columns)

        return (self.columns, data)


class DeleteCCECluster(command.Command):
    _description = _('Delete CCE Cluster')

    def get_parser(self, prog_name):
        parser = super(DeleteCCECluster, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('Name or ID of the cluster')
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help=_('Wait for the instance to become active')
        )
        parser.add_argument(
            '--wait-interval',
            type=int,
            help=_('Interval for checking status')
        )
        parser.add_argument(
            '--wait-timeout',
            type=int,
            help=_('Wait timeout')
        )
        return parser

    def take_action(self, parsed_args):

        attrs = {}
        for attr in [
            'cluster', 'wait', 'wait_timeout', 'wait_interval'
        ]:
            if getattr(parsed_args, attr):
                attrs[attr] = getattr(parsed_args, attr)

        if not parsed_args.wait:
            attrs['wait'] = False

        # initialize sdk_connection with cce methods
        _ = self.app.client_manager.cce
        if parsed_args.cluster:
            self.app.client_manager.sdk_connection.delete_cce_cluster(
                **attrs
            )


class CreateCCECluster(command.ShowOne):
    _description = _('Create CCE Cluster')
    columns = ('ID', 'name', 'version', 'endpoint')

    def get_parser(self, prog_name):
        parser = super(CreateCCECluster, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of the cluster.')
        )
        parser.add_argument(
            'router',
            metavar='<router>',
            help=_('ID or name the Neutron Router.')
        )
        parser.add_argument(
            'network',
            metavar='<network>',
            help=_('ID or name of the Neutron network.')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Text description of the cluster.')
        )
        parser.add_argument(
            '--multi-az',
            action='store_true',
            help=_('Support for multi-AZ cluster')
        )
        parser.add_argument(
            '--version',
            metavar='<version>',
            help=_('Cluster version.')
        )
        parser.add_argument(
            '--flavor',
            metavar='<flavor>',
            required=True,
            help=_('Cluster flavor.')
        )
        parser.add_argument(
            '--type',
            metavar='{' + ','.join(CLUSTER_TYPES) + '}',
            choices=CLUSTER_TYPES,
            default='VirtualMachine',
            help=_('Cluster type.')
        )
        parser.add_argument(
            '--container-network-mode',
            metavar='{' + ','.join(CONTAINER_NET_MODE_CHOICES) + '}',
            type=lambda s: s.lower(),
            choices=CONTAINER_NET_MODE_CHOICES,
            default='overlay_l2',
            help=_('Container network mode.')
        )
        parser.add_argument(
            '--container-network-cidr',
            metavar='<CIDR>',
            help=_('Network segment of the container network.')
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help=('Wait for the instance to become active')
        )
        parser.add_argument(
            '--wait-interval',
            type=int,
            help=_('Interval for checking status')
        )
        parser.add_argument(
            '--wait-timeout',
            type=int,
            help=_('Wait timeout')
        )
        return parser

    def take_action(self, parsed_args):

        attrs = {}
        for attr in [
            'name', 'type', 'flavor', 'router', 'network',
            'container_network_mode', 'annotations', 'authentication_proxy_ca',
            'authentication_mode', 'container_network_cidr',
            'cpu_manager_policy', 'dss_master_volumes', 'description',
            'external_ip', 'fix_pool_mask', 'labels', 'service_ip_range',
            'kube_proxy_mode', 'upgrade_from', 'version', 'wait',
            'wait_timeout', 'wait_interval'
        ]:
            if hasattr(parsed_args, attr) and getattr(parsed_args, attr):
                attrs[attr] = getattr(parsed_args, attr)

        if not parsed_args.wait:
            attrs['wait'] = False
        if parsed_args.multi_az:
            attrs['az'] = 'multi_az'

        # initialize sdk_connection with cce methods
        _ = self.app.client_manager.cce
        obj = self.app.client_manager.sdk_connection.create_cce_cluster(
            **attrs)

        data = utils.get_dict_properties(_flatten_cluster(obj), self.columns)

        return (self.columns, data)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""Create a devenv instance from attributes."""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud="otc")

attrs = {
    "name": "notebook-d115",
    "flavor": "modelarts.vm.cpu.2u",
    "spec": {
        "storage": {
            "location": {
                "path": "/test-bucket/notebooks/",
            },
            "type": "obs",
        }
    },
    "profile_id": "Multi-Engine 1.0 (python3)-cpu",
}

devenv_instance = conn.modelartsv1.create_devenv_instance(**attrs)
print(devenv_instance)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource
from openstack import utils

from otcextensions.sdk.dms.v1 import _base


class Group(_base.Resource):

    # NOTE: we are not interested in the also returned short queue info
    resources_key = 'groups'
    base_path = 'queues/%(queue_id)s/groups'

    # capabilities
    allow_create = True
    allow_list = True
    allow_delete = True

    _query_mapping = resource.QueryParameters(
        'include_deadletter', 'page_size', 'current_page'
        # 'include_messages_num'
    )

    # Properties
    queue_id = resource.URI('queue_id')

    #: Consume group Id
    id = resource.Body('id')
    #: Name
    name = resource.Body('name')
    #: Total message number, not including deleted message
    #: *Type: int*
    produced_messages = resource.Body('produced_messages', type=int)
    #: Consumed message number
    #: *Type: int*
    consumed_messages = resource.Body('consumed_messages', type=int)
    #: Available message number
    #: *Type: int*
    available_messages = resource.Body('available_messages', type=int)
    #: Total deadletters number
    #: *Type: int*
    produced_deadletters = resource.Body('produced_deadletters', type=int)
    #: Available deadletters number
    #: *Type: int*
    available_deadletters = resource.Body('available_deadletters', type=int)

    def create(self, session, *args, **kwargs):
        """create group"""
        body = {"groups": [{'name': self.name}]}

        request = self._prepare_request(requires_id=False,
                                        prepend_key=False)

        response = session.post(
            request.url,
            json=body
        )

        # Squize groups into single response entity
        resp = response.json()
        if self.resources_key in resp:
            res = resp[self.resources_key][0]

            res = self._consume_body_attrs(res)
            self._body.attributes.update(res)
            self._body.clean()

        return self

    def ack(self, session, queue_obj, ids, status='success'):
        uri = utils.urljoin(self.base_path, self.id, 'ack')
        self.queue_id = queue_obj.id

        uri = uri % self._uri.attributes

        ack_list = list()
        for msg in ids:
            ack_list.append(
                {"handler": msg,
                 "status": status}
            )

        response = session.post(
            uri, json={'message': ack_list}
        )

        exceptions.raise_from_response(response)
        return



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from otcextensions.osclient.dms.v1 import topic
from otcextensions.tests.unit.osclient.dms.v1 import fakes


class TestDMSInstanceTopic(fakes.TestDMS):

    def setUp(self):
        super(TestDMSInstanceTopic, self).setUp()
        self.client = self.app.client_manager.dms
        self.instance = fakes.FakeInstance.create_one()
        self.client.find_instance = mock.Mock(return_value=self.instance)


class TestListDMSInstanceTopic(TestDMSInstanceTopic):

    topics = fakes.FakeTopic.create_multiple(3)

    columns = ('ID', 'replication', 'partition', 'retention_time',
               'is_sync_flush', 'is_sync_replication')

    data = []

    for s in topics:
        data.append((
            s.id,
            s.replication,
            s.partition,
            s.retention_time,
            s.is_sync_flush,
            s.is_sync_replication
        ))

    def setUp(self):
        super(TestListDMSInstanceTopic, self).setUp()

        self.cmd = topic.ListDMSInstanceTopic(self.app, None)

        self.client.topics = mock.Mock()

    def test_list_topics(self):
        arglist = [
            'inst'
        ]

        verifylist = [
            ('instance', 'inst')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.topics.side_effect = [
            self.topics
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_instance.assert_called_once_with(
            'inst', ignore_missing=False)
        self.client.topics.assert_called_once_with(instance=self.instance)

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestDeleteDMSInstanceTopic(TestDMSInstanceTopic):

    def setUp(self):
        super(TestDeleteDMSInstanceTopic, self).setUp()

        self.cmd = topic.DeleteDMSInstanceTopic(self.app, None)

        self.client.delete_topic = mock.Mock()

    def test_delete(self):
        arglist = ['inst', 't1']
        verifylist = [
            ('instance', 'inst'),
            ('topic', ['t1'])
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_instance.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        calls = [mock.call(instance=self.instance, topics=['t1'])]

        self.client.find_instance.assert_called_with(
            'inst', ignore_missing=False)
        self.client.delete_topic.assert_has_calls(calls)
        self.assertEqual(1, self.client.delete_topic.call_count)

    def test_delete_multiple(self):
        arglist = [
            'inst',
            't1',
            't2',
        ]
        verifylist = [
            ('instance', 'inst'),
            ('topic', ['t1', 't2'])
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_instance.side_effect = [{}, {}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        calls = [mock.call(instance=self.instance, topics=['t1', 't2'])]

        self.client.find_instance.assert_called_with(
            'inst', ignore_missing=False)
        self.client.delete_topic.assert_has_calls(calls)
        self.assertEqual(1, self.client.delete_topic.call_count)


class TestCreateDMSInstanceTopic(TestDMSInstanceTopic):

    _data = fakes.FakeTopic.create_one()

    columns = ('id', 'is_sync_flush', 'is_sync_replication', 'partition',
               'replication', 'retention_time')

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestCreateDMSInstanceTopic, self).setUp()

        self.cmd = topic.CreateDMSInstanceTopic(self.app, None)

        self.client.create_topic = mock.Mock()

    def test_create_default(self):
        arglist = [
            'inst',
            'topic',
            '--partition', '5',
            '--replication', '3',
            '--retention-time', '8',
            '--enable-sync-flush',
            '--enable-sync-replication'
        ]
        verifylist = [
            ('instance', 'inst'),
            ('id', 'topic'),
            ('partition', 5),
            ('replication', 3),
            ('retention_time', 8),
            ('enable_sync_flush', True),
            ('enable_sync_replication', True),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_topic.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_topic.assert_called_with(
            instance=self.instance,
            id='topic',
            is_sync_flush=True,
            is_sync_replication=True,
            partition=5,
            replication=3,
            retention_time=8
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#

BUILT_IN_MODEL = {
    "model_id": 4,
    "model_name": "ResNet_v2_50",
    "model_usage": 1,
    "model_precision": "75.55%(top1), 92.6%(top5)",
    "model_size": 102503801,
    "model_train_dataset": "ImageNet, 1,000 classes for image classification",
    "model_dataset_format": "shape: [H>=32, W>=32, C>=1]; type: int8",
    "model_description_url": "https://github.com/....../resnet.py",
    "parameter": [
        {
            "label": "batch_size",
            "value": "4",
            "required": True,
        }
    ],
    "create_time": 1522218780025,
    "engine_id": 501,
    "engine_name": "MXNet",
    "engine_version": "MXNet-1.2.1-python2.7",
}

DEVENV = {
    "ai_project": {"id": "default-ai-project"},
    "creation_timestamp": "1686643651085",
    "description": "",
    "flavor": "modelarts.vm.cpu.2u",
    "flavor_details": {
        "name": "modelarts.vm.cpu.2u",
        "params": {
            "CPU": 2,
            "GPU": 0,
            "memory": "8GiB",
        },
        "params_extends": {
            "arch": "X86_64",
            "cpu": {
                "memory_size": "8",
                "memory_unit": "GB",
                "num": 2,
            },
        },
        "status": "onSale",
        "type": "CPU",
    },
    "id": "DE-uuid",
    "latest_update_timestamp": "1709117152833",
    "name": "fsd",
    "profile": {
        "de_type": "Notebook",
        "description": "multi engine, cpu, python 3.6 for notebook",
        "id": "eb01a452-7fb2-11ed-b1b8-0255c0a80049",
        "name": "Multi-Engine 2.0 (python3)",
        "provision": {
            "annotations": {
                "category": "Multi-Engine 2.0 (Python3)",
                "type": "system",
            },
            "spec": {
                "engine": "CCE",
                "params": {
                    "annotations": None,
                    "image_name": "mul-kernel2.0-cp36",
                    "image_tag": "3.2.0-latest-p2",
                    "namespace": "atelier",
                },
            },
            "type": "Docker",
        },
    },
    "spec": {
        "annotations": {
            "target_domain": "https://notebook.eu-de.otc.t-systems.com",
            "url": "https://...",
        },
        "auto_stop": {
            "duration": 3600,
            "enable": True,
            "prompt": True,
            "remain_time": 3336,
            "stop_timestamp": 1594891408723,
        },
        "extend_params": None,
        "extend_storage": None,
        "failed_reasons": None,
        "repository": None,
        "storage": {
            "location": {
                "path": "/home/ma-user/work",
                "volume_size": 5,
                "volume_unit": "GB",
            },
            "type": "evs",
        },
    },
    "status": "RUNNING",
    "user": {
        "id": "user-uuid",
        "name": "ops_dev_env",
    },
    "workspace": {
        "id": "0",
    },
}


JOB_FLAVOR = {
    "spec_id": 44,
    "core": "8",
    "cpu": "32",
    "gpu_num": 0,
    "gpu_type": "",
    "spec_code": "modelarts.vm.cpu.8u",
    "unit_num": 1,
    "max_num": 1,
    "storage": "",
    "interface_type": 1,
    "no_resource": False,
}


MODEL = {
    "ai_project": "default-ai-project",
    "apis": [
        {
            "method": "post",
            "url": "/",
            "input_params": {
                "type": "object",
                "properties": {"images": {"type": "file"}},
            },
            "output_params": {
                "type": "object",
                "required": [
                    "detection_classes",
                    "detection_boxes",
                    "detection_scores",
                ],
                "properties": {
                    "detection_classes": {
                        "type": "array",
                        "item": {"type": "string"},
                    },
                    "detection_boxes": {
                        "type": "array",
                        "items": {
                            "type": "array",
                            "minItems": 4.0,
                            "maxItems": 4.0,
                            "items": {"type": "number"},
                        },
                    },
                    "detection_scores": {
                        "type": "array",
                        "item": {"type": "number"},
                    },
                },
            },
            "content_type": "multipart/form-data",
        }
    ],
    "config": "{...}",
    "create_at": 1673863048731,
    "dependencies": [
        {
            "installer": "pip",
            "packages": [
                {
                    "package_name": "numpy",
                    "package_version": "1.17.0",
                    "restraint": "EXACT",
                },
                {
                    "package_name": "h5py",
                    "package_version": "2.8.0",
                    "restraint": "EXACT",
                },
                {
                    "package_name": "Pillow",
                    "package_version": "5.2.0",
                    "restraint": "EXACT",
                },
                {
                    "package_name": "scipy",
                    "package_version": "1.2.1",
                    "restraint": "EXACT",
                },
                {
                    "package_name": "resampy",
                    "package_version": "0.2.1",
                    "restraint": "EXACT",
                },
                {
                    "package_name": "scikit-learn",
                    "package_version": "0.22.2",
                    "restraint": "EXACT",
                },
            ],
        }
    ],
    "execution_code": "https://...",
    "image_address": "swr.eu-de.otc.t-systems.com/...",
    "input_params": [
        {
            "url": "/",
            "method": "post",
            "param_name": "images",
            "param_type": "file",
            "param_desc": '{"type":"file"}',
        }
    ],
    "install_type": ["real-time", "batch"],
    "market_flag": False,
    "model_algorithm": "object_detection",
    "model_id": "model-id",
    "model_metrics": "{...}",
    "model_name": "MA_UAT_002_ExeML_c2dd3f46",
    "model_size": 175791900,
    "model_source": "auto",
    "model_status": "published",
    "model_type": "TensorFlow",
    "model_version": "0.2.6",
    "output_params": [
        {
            "url": "/",
            "method": "post",
            "param_name": "detection_classes",
            "param_type": "array",
            "param_desc": '{"type":"array","item":{"type":"string"}}',
        },
        {
            "url": "/",
            "method": "post",
            "param_name": "detection_scores",
            "param_type": "array",
            "param_desc": '{"type":"array","item":{"type":"number"}}',
        },
    ],
    "owner": "owner-id",
    "project": "project-id",
    "publishable_flag": False,
    "runtime": "tf1.13-python3.7-cpu",
    "source_location": "https://train_url...",
    "source_type": "auto",
    "specification": {},
    "tenant": "tenant-id",
    "tunable": False,
    "workspace_id": "0",
}

SERVICE = {
    "access_address": "https://access_address",
    "additional_properties": {},
    "config": [
        {
            "model_id": "model-uuid",
            "model_name": "model-name",
            "model_version": "0.0.1",
            "source_type": "auto",
            "specification": "modelarts.vm.high.p3",
            "custom_spec": {},
            "status": "ready",
            "weight": 100,
            "instance_count": 1,
            "scaling": False,
            "envs": {},
            "additional_properties": {},
            "support_debug": False,
        }
    ],
    "description": "Created by Exeml project(name: exeML-aqi).",
    "failed_times": 3,
    "infer_type": "real-time",
    "invocation_times": 5,
    "is_free": False,
    "is_shared": False,
    "operation_time": 1705621491068,
    "owner": "owner-uuid",
    "progress": 100,
    "project": "project-uuid",
    "publish_at": 1691908601470,
    "service_id": "service-uuid",
    "service_name": "exeML-aqi_ExeML_1691908601350529540",
    "shared_count": 0,
    "status": "running",
    "tenant": "tenant-uuid",
    "transition_at": 1696333731658,
    "update_time": 1691908601470,
    "workspace_id": "0",
}

SERVICE_CLUSTER = {
    "cluster_id": "cluster-uuid",
    "cluster_name": "pool-a1cf",
    "tenant": "tenant-uuid",
    "project": "project-uuid",
    "owner": "owner-uuid",
    "created_at": 1658743383618,
    "status": "running",
    "allocatable_cpu_cores": 7.06,
    "allocatable_memory": 27307.0,
    "allocatable_gpus": 0.0,
    "charging_mode": "postpaid",
    "max_node_count": 50,
    "nodes": {
        "specification": "modelarts.vm.cpu.8ud",
        "count": 1,
        "available_count": 1,
    },
    "services_count": {"realtime_count": 0, "batch_count": 0},
}

SERVICE_EVENT = {
    "occur_time": 1562597251764,
    "event_type": "normal",
    "event_info": "start to deploy service",
}

SERVICE_FLAVOR = {
    "specification": "modelarts.vm.gpu.v100",
    "billing_spec": "modelarts.vm.gpu.v100",
    "is_open": True,
    "spec_status": "normal",
    "is_free": False,
    "over_quota": False,
    "extend_params": 1,
}

SERVICE_LOG = {
    "config": [
        {
            "model_id": "f565ac47-6239-4e8c-b2dc-0665dc52e302",
            "model_name": "model-demo",
            "model_version": "0.0.1",
            "specification": "modelarts.vm.cpu.2u",
            "custom_spec": {},
            "weight": 100,
            "instance_count": 1,
            "scaling": False,
            "envs": {},
            "cluster_id": "2c9080f86d37da64016d381fe5940002",
        }
    ],
    "extend_config": [],
    "update_time": 1586250930708,
    "result": "RUNNING",
    "cluster_id": "2c9080f86d37da64016d381fe5940002",
}

SERVICE_MONITOR = {
    "model_id": "xxxx",
    "model_name": "minst",
    "model_version": "1.0.0",
    "invocation_times": 50,
    "failed_times": 1,
    "cpu_core_usage": 2.4,
    "cpu_core_total": 4,
    "cpu_memory_usage": 2011,
    "cpu_memory_total": 8192,
    "gpu_usage": 0.6,
    "gpu_total": 1,
}

TRAINING_JOB = {
    "job_id": 9446,
    "job_name": "ma-testing",
    "version_id": 10818,
    "status": 10,
    "create_time": 1704448084000,
    "duration": 54793000,
    "job_desc": "This is a ModelArts job",
    "version_count": 1,
    "user": {"name": "userAbc"},
}

TRAINING_JOB_CONFIG = {
    "spec_code": "modelarts.vm.gpu.v100",
    "user_image_url": "100.125.5.235:20202/jobmng/custom-cpu-base:1.0",
    "user_command": "bash -x /home/work/run_train.sh python \
                /home/work/user-job-dir/app/mnist/mnist_softmax.py \
                --data_url /home/work/user-job-dir/app/mnist_data",
    "dataset_version_id": "2ff0d6ba-c480-45ae-be41-09a8369bfc90",
    "engine_name": "TensorFlow",
    "is_success": True,
    "nas_mount_path": "/home/work/nas",
    "worker_server_num": 1,
    "nas_share_addr": "192.168.8.150:/",
    "train_url": "/test/minst/train_out/out1/",
    "nas_type": "nfs",
    "spec_id": 4,
    "parameter": [{"label": "learning_rate", "value": 0.01}],
    "log_url": "/usr/log/",
    "config_name": "config123",
    "app_url": "/usr/app/",
    "create_time": 1559045426000,
    "dataset_id": "38277e62-9e59-48f4-8d89-c8cf41622c24",
    "volumes": [
        {
            "nfs": {
                "id": "43b37236-9afa-4855-8174-32254b9562e7",
                "src_path": "192.168.8.150:/",
                "dest_path": "/home/work/nas",
                "read_only": False,
            }
        },
        {
            "host_path": {
                "src_path": "/root/work",
                "dest_path": "/home/mind",
                "read_only": False,
            }
        },
    ],
    "cpu": "64",
    "model_id": 4,
    "boot_file_url": "/usr/app/boot.py",
    "dataset_name": "dataset-test",
    "pool_id": "pool9928813f",
    "config_desc": "This is a config desc test",
    "gpu_num": 1,
    "data_source": [{"type": "obs", "data_url": "/test/minst/data/"}],
    "pool_name": "p100",
    "dataset_version_name": "dataset-version-test",
    "core": "8",
    "engine_type": 1,
    "engine_id": 3,
    "engine_version": "TF-1.8.0-python2.7",
    "data_url": "/test/minst/data/",
}

TRAINING_JOB_VERSION = {
    "job_id": 9275,
    "job_name": "matt-billing-retest-001",
    "job_desc": "test",
    "version_id": 10617,
    "version_name": "V0001",
    "pre_version_id": None,
    "resource_id": "jobbc4a1f13",
    "status": 10,
    "app_url": None,
    "boot_file_url": None,
    "create_time": 1701698401000,
    "start_time": 1701698668000,
    "is_zombie": None,
    "parameter": [
        {
            "label": "split_spec",
            "value": "train:0.8,eval:0.2",
            "required": True,
            "placeholder": "",
            "tip": {
                "content": "",
                "position": "top-left",
                "show": False,
            },
        },
    ],
    "duration": 355000,
    "spec_id": 5,
    "user_image_url": None,
    "user_command": None,
    "worker_server_num": 1,
    "data_url": "/ma-test-001/annotation/V002/V002.manifest",
    "train_url": "/matt-rgbtest/matt-billing-retest-001/",
    "dataset_id": "6KuaOYmLRz9tA2UrP4H",
    "dataset_version_id": "1EZAJDvxCuOcCqQ47ep",
    "dataset_version_name": "V002",
    "data_source": None,
    "dataset_name": "MA-UAT-001",
    "model_id": 1,
    "model_metric_list": {
        "total_metric": {
            "total_reserved_data": {},
            "total_metric_meta": {},
            "total_metric_values": {
                "recall": 0.996078,
                "precision": 0.995918,
                "f1_score": 0.995958,
                "accuracy": 0.996094,
                "top-1": 0.996094,
                "top-5": 1,
            },
        },
        "metric": [
            {
                "metric_meta": {
                    "class_name": "tulips",
                    "class_id": 4,
                },
                "metric_values": {
                    "recall": 0.980392,
                    "precision": 1,
                    "f1_score": 0.990099,
                },
                "reserved_data": {},
            },
        ],
        "images_outputs": {
            "s3://test-bucket/tulips/141479422_5a6fa1fd1b_m.jpg": {
                "real_label": ["tulips"],
                "logits": [
                    "8.085395",
                    "0.010677",
                    "-1.837408",
                    "-3.062298",
                    "-4.201297",
                ],
                "probability": [
                    "0.999621",
                    "0.000311",
                    "0.000049",
                    "0.000014",
                    "0.000005",
                ],
                "predict_label": [
                    "tulips",
                    "roses",
                    "daisy",
                    "dandelion",
                    "sunflowers",
                ],
            },
        },
    },
    "system_metric_list": {
        "interval": 1,
        "memUsage": ["0", "0", "3.58", "6.26", "11.57", "13.11", "0"],
        "recvBytesRate": [
            "0",
            "0",
            "5667968.00",
            "9162228.00",
            "9627428.00",
            "14266488.00",
            "0",
        ],
        "diskWriteRate": ["0", "0", "1.00", "0.81", "0.61", "0.31", "0"],
        "cpuUsage": ["0", "0", "18.92", "23.45", "32.25", "38.97", "0"],
        "sendBytesRate": [
            "0",
            "0",
            "1930724.00",
            "10253752.00",
            "18480064.00",
            "7347611.50",
            "0",
        ],
        "gpuUtil": ["0", "0", "0.40", "9.50", "32.50", "52.00", "0"],
        "gpuMemUsage": ["0", "0", "56.10", "70.35", "94.11", "94.11", "0"],
        "diskReadRate": [
            "0",
            "0",
            "42159.50",
            "23886.87",
            "9.00",
            "0.00",
            "0",
        ],
    },
    "log_url": None,
    "nas_type": "efs",
    "nas_share_addr": None,
    "nas_mount_path": None,
    "retrain_model_id": None,
    "volumes": None,
    "algorithm_id": None,
    "core": "8",
    "cpu": "64",
    "memory_unit": "GB",
    "gpu_num": 1,
    "gpu_type": "nvidia-v100-pcie",
    "gpu_memory_unit": "GB",
    "job_type": 1,
    "spec_code": "modelarts.vm.gpu.v100NV",
    "max_num": 1,
    # "storage": "",
    "no_resource": False,
    "npu_info": None,
    "engine_id": 500,
    "engine_type": 1000,
    "engine_name": "INNER_TensorFlow",
    "engine_version": "TF-1.8.0-python2.7",
    "pod_version": "1.8.0-cp27",
    "pool_id": "pool7e3577e3",
    "pool_name": "job-train-1v100",
    "flavor_code": "modelarts.vm.gpu.1v100NV32.pub",
    "pool_type": "SYSTEM_DEFINED",
    "flavor_type": "GPU",
    "billing": {"unit_num": 1, "code": "modelarts.vm.gpu.v100NV"},
    "flavor_info": {
        "cpu": {"arch": "x86", "core_num": 8},
        "ram": {"memory": 64, "unit": "GB"},
        "gpu": {
            "unit_num": 1,
            "product_name": "NVIDIA-V100-pcie",
            "memory": "16",
            "unit": "GB",
        },
        "max_num": 1,
        "npu_info": "",
        "npu": {
            "product_name": "",
            "unit": "GB",
            "unit_num": 0,
            "memory": 0,
            "info": "",
        },
    },
    "attributes": {"isSupportDist": "True"},
    "description": "",
    "is_free": False,
    "is_success": True,
}

VISUALIZATION_JOB = {
    "duration": 33000,
    "service_url": "https://...",
    "job_name": "apiTest-11",
    "create_time": 1565149736000,
    "train_url": "/wph-test/zl-test/Flowers-Set/ApiTest/",
    "job_id": 197,
    "job_desc": "ModelArts API Dialtest",
    "resource_id": "e17dd874-b5e0-4e9b-aaf0-22b045ad8571",
    "remaining_duration": None,
    "is_success": True,
    "status": 7,
}



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''CCE Cluster v1 action implementations'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _flatten_cluster(obj):
    data = {
        'id': obj.id,
        'name': obj.name,
        # 'status': obj.status['status'],
        'cpu': obj.spec.cpu,
        'memory': obj.spec.memory,
        'endpoint': obj.spec.endpoint,
        'availability_zone': obj.spec.availability_zone,
        'vpc': obj.spec.vpc,
        # 'nodes': len(obj.spec.host_list.spec.host_list)
    }
    if obj.status:
        data['status'] = obj.status['status']
    try:
        data['nodes'] = len(obj.spec.host_list.spec.host_list)
    except (TypeError, AttributeError):
        data['nodes'] = 0

    return data


class ListCCECluster(command.Lister):
    _description = _('List CCE Clusters')
    columns = ('ID', 'name', 'cpu', 'memory', 'endpoint')

    def get_parser(self, prog_name):
        parser = super(ListCCECluster, self).get_parser(prog_name)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cce

        data = client.clusters()

        table = (self.columns,
                 (utils.get_dict_properties(
                     _flatten_cluster(s), self.columns,
                 ) for s in data))
        return table


class ShowCCECluster(command.ShowOne):
    _description = _('Show single Cluster details')
    columns = ('ID', 'name', 'status', 'cpu', 'memory', 'endpoint',
               'availability_zone',
               'vpc', 'nodes')

    def get_parser(self, prog_name):
        parser = super(ShowCCECluster, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID of the cluster')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cce

        obj = client.find_cluster(parsed_args.cluster)

        # display_columns, columns = _get_columns(obj)
        data = utils.get_dict_properties(_flatten_cluster(obj), self.columns)

        return (self.columns, data)


class DeleteCCECluster(command.Command):
    _description = _('Delete CCE Cluster')

    def get_parser(self, prog_name):
        parser = super(DeleteCCECluster, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID of the cluster')
        )
        return parser

    def take_action(self, parsed_args):

        if parsed_args.cluster:
            client = self.app.client_manager.cce
            client.delete_cluster(parsed_args.cluster)


class CreateCCECluster(command.Command):
    _description = _('Create CCE Cluster')
    columns = ('ID', 'name', 'status', 'cpu', 'memory', 'endpoint',
               'availability_zone',
               'vpc', 'nodes')

    def get_parser(self, prog_name):
        parser = super(CreateCCECluster, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of the cluster.')
        )
        parser.add_argument(
            'vpc',
            metavar='<vpc>',
            help=_('The VPC ID of the container cluster.')
        )
        parser.add_argument(
            'subnet',
            metavar='<subnet>',
            help=_('The Subnet ID of the container cluster.')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Text description of the cluster.')
        )
        parser.add_argument(
            '--region',
            metavar='<region>',
            required=True,
            help=_('The Region to which container cluster belongs.')
        )
        parser.add_argument(
            '--security_group',
            metavar='<security_group>',
            help=_('Security group ID of the container cluster.\n'
                   'If this parameter is left blank, CCE will '
                   'dynamically create a security group when '
                   'creating the container cluster.')
        )
        parser.add_argument(
            '--type',
            metavar='<type>',
            choices=['HA', 'Single'],
            help=_('Container cluster type. '
                   'One of [`HA`, `Single`]. Default: `Single`')
        )
        return parser

    def take_action(self, parsed_args):

        attrs = {}

        attrs['name'] = parsed_args.name
        spec = {}
        spec['vpc'] = parsed_args.vpc
        spec['subnet'] = parsed_args.subnet
        spec['region'] = parsed_args.region
        if parsed_args.description:
            spec['description'] = parsed_args.description
        if parsed_args.security_group:
            spec['security_group_id'] = parsed_args.security_group
        if parsed_args.type:
            spec['cluster_type'] = parsed_args.type

        attrs['spec'] = spec

        client = self.app.client_manager.cce

        obj = client.create_cluster(**attrs)

        data = utils.get_dict_properties(_flatten_cluster(obj), self.columns)

        return (self.columns, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import datetime
import functools
import hashlib
import hmac

import requests

from urllib.parse import quote
from urllib.parse import urlparse
from urllib.parse import urlsplit

from http import client as http_client


class HTTPHeaders(http_client.HTTPMessage):
    pass


def ensure_unicode(s, encoding=None, errors=None):
    # NOOP in Python 3, because every string is already unicode
    return s


EMPTY_SHA256_HASH = (
    'e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855')

UNSIGNED_PAYLOAD = 'UNSIGNED-PAYLOAD'
PAYLOAD_BUFFER = 1024 * 1024

SIGNED_HEADERS_BLACKLIST = [
    'expect',
    'user-agent',
    'x-amzn-trace-id',
    'X-Auth-Token',
]
ISO8601 = '%Y-%m-%dT%H:%M:%SZ'
SIGV4_TIMESTAMP = '%Y%m%dT%H%M%SZ'


class AKRequestsAuth(requests.auth.AuthBase):
    """
    Auth class that allows us to connect to AWS services
    via Amazon's signature version 4 signing process
    Adapted from
        https://docs.aws.amazon.com/general/latest/gr/sigv4-signed-request-examples.html
    """
    IDENTITY_AUTH_HEADER_NAME = 'X-Amz-Security-Token'

    def __init__(self,
                 access_key,
                 secret_access_key,
                 host,
                 region,
                 service,
                 token=None):
        """
        Example usage for talking to an AWS Elasticsearch Service:
        AKRequestsAuth(aws_access_key='YOURKEY',
               aws_secret_access_key='YOURSECRET',
               aws_host='search-service-foobar.us-east-1.es.amazonaws.com',
               aws_region='us-east-1',
               aws_service='es',
               aws_token='...')
        The aws_token is optional and is used only if you are using STS
        temporary credentials.
        """
        self.aws_access_key = access_key
        self.aws_secret_access_key = secret_access_key
        self.aws_host = host
        self.aws_region = region
        self.service = service
        self.aws_token = token

    def __call__(self, r):
        """
        Adds the authorization headers required by Amazon's signature
        version 4 signing process to the request.
        Adapted from
            https://docs.aws.amazon.com/general/latest/gr/sigv4-signed-request-examples.html
        """
        self.add_auth(r)
        return r

    def add_auth(self, request):
        """
        Returns a dictionary containing the necessary headers for Amazon's
        signature version 4 signing process. An example return value might
        look like
        {
            'Authorization': 'AWS4-HMAC-SHA256 Credential=YOURKEY/20160618'
                             '/us-east-1/es/aws4_request, '
                             'SignedHeaders=host;x-amz-date, '
                             'Signature=ca0a856286efce2a4bd96a978ca6c8966057e53184776c0685169d08abd74739',
            'x-amz-date': '20160618T220405Z',
        }
        """

        datetime_now = datetime.datetime.utcnow()
        self.timestamp = datetime_now.strftime(SIGV4_TIMESTAMP)
        # This could be a retry.  Make sure the previous
        # authorization header is removed first.
        self._modify_request_before_signing(request)
        canonical_request = self.canonical_request(request)
        string_to_sign = self.string_to_sign(request, canonical_request)
        signature = self.signature(string_to_sign, request)

        self._inject_signature_to_request(request, signature)

    def _modify_request_before_signing(self, request):
        if 'Authorization' in request.headers:
            del request.headers['Authorization']
        self._set_necessary_date_headers(request)
        if self.aws_token:
            if 'X-Amz-Security-Token' in request.headers:
                del request.headers['X-Amz-Security-Token']
            request.headers['X-Amz-Security-Token'] = self.aws_token

        # if not self._should_sha256_sign_payload(request):
        if 'X-Amz-Content-SHA256' in request.headers:
            del request.headers['X-Amz-Content-SHA256']
        request.headers['X-Amz-Content-SHA256'] = UNSIGNED_PAYLOAD

    def _set_necessary_date_headers(self, request):
        # The spec allows for either the Date _or_ the X-Amz-Date value to be
        # used so we check both.  If there's a Date header, we use the date
        # header.  Otherwise we use the X-Amz-Date header.
        # if 'Date' in request.headers:
        #     del request.headers['Date']
        #     datetime_timestamp = datetime.datetime.strptime(
        #         self.timestamp, SIGV4_TIMESTAMP)
        #     request.headers['Date'] = formatdate(
        #         int(calendar.timegm(datetime_timestamp.timetuple())))
        #     if 'X-Amz-Date' in request.headers:
        #         del request.headers['X-Amz-Date']
        # else:
        if 'X-Amz-Date' in request.headers:
            del request.headers['X-Amz-Date']
        request.headers['X-Amz-Date'] = self.timestamp

    def _inject_signature_to_request(self, request, signature):
        hdrs = ['AWS4-HMAC-SHA256 Credential=%s' % self.scope(request)]
        headers_to_sign = self.headers_to_sign(request)
        hdrs.append('SignedHeaders=%s' % self.signed_headers(headers_to_sign))
        hdrs.append('Signature=%s' % signature)
        request.headers['Authorization'] = ', '.join(hdrs)
        return request

    def payload(self, request):
        if not self._should_sha256_sign_payload(request):
            # When payload signing is disabled, we use this static string in
            # place of the payload checksum.
            return UNSIGNED_PAYLOAD
        if request.body and hasattr(request.body, 'seek'):
            position = request.body.tell()
            read_chunksize = functools.partial(request.body.read,
                                               PAYLOAD_BUFFER)
            checksum = hashlib.sha256()
            for chunk in iter(read_chunksize, b''):
                checksum.update(chunk)
            hex_checksum = checksum.hexdigest()
            request.body.seek(position)
            return hex_checksum
        elif request.body:
            # The request serialization has ensured that
            # request.body is a bytes() type.
            return hashlib.sha256(request.body).hexdigest()
        else:
            return EMPTY_SHA256_HASH

    def _should_sha256_sign_payload(self, request):
        # Payloads will always be signed over insecure connections.
        if not request.url.startswith('https'):
            return True

        # Certain operations may have payload signing disabled by default.
        # Since we don't have access to the operation model, we pass in this
        # bit of metadata through the request context.
        return True  # request.context.get('payload_signing_enabled', True)

    def canonical_headers(self, headers_to_sign):
        """
        Return the headers that need to be included in the StringToSign
        in their canonical form by converting all header keys to lower
        case, sorting them in alphabetical order and then joining
        them into a string, separated by newlines.
        """
        headers = []
        sorted_header_names = sorted(set(headers_to_sign))
        for key in sorted_header_names:
            value = ','.join(self._header_value(v) for v in
                             sorted(headers_to_sign.get_all(key)))
            headers.append('%s:%s' % (key, ensure_unicode(value)))
        return '\n'.join(headers)

    def _header_value(self, value):
        # From the sigv4 docs:
        # Lowercase(HeaderName) + ':' + Trimall(HeaderValue)
        #
        # The Trimall function removes excess white space before and after
        # values, and converts sequential spaces to a single space.
        return ' '.join(value.split())

    @classmethod
    def get_canonical_path(cls, r):
        """
        Create canonical URI--the part of the URI from domain to query
        string (use '/' if no path)
        """
        parsedurl = urlparse(r.url)

        # safe chars adapted from boto's use of urllib.parse.quote
        # https://github.com/boto/boto/blob/d9e5cfe900e1a58717e393c76a6e3580305f217a/boto/auth.py#L393
        return quote(parsedurl.path if parsedurl.path else '/', safe='/-_.~')

    @classmethod
    def get_canonical_querystring(cls, r):
        """
        Create the canonical query string. According to AWS, by the
        end of this function our query string values must
        be URL-encoded (space=%20) and the parameters must be sorted
        by name.
        This method assumes that the query params in `r` are *already*
        url encoded.  If they are not url encoded by the time they make
        it to this function, AWS may complain that the signature for your
        request is incorrect.
        It appears elasticsearc-py url encodes query paramaters on its own:
            https://github.com/elastic/elasticsearch-py/blob/5dfd6985e5d32ea353d2b37d01c2521b2089ac2b/elasticsearch/connection/http_requests.py#L64
        If you are using a different client than elasticsearch-py, it
        will be your responsibility to urleconde your query params before
        this method is called.
        """
        canonical_querystring = ''

        parsedurl = urlparse(r.url)
        querystring_sorted = '&'.join(sorted(parsedurl.query.split('&')))

        for query_param in querystring_sorted.split('&'):
            key_val_split = query_param.split('=', 1)

            key = key_val_split[0]
            if len(key_val_split) > 1:
                val = key_val_split[1]
            else:
                val = ''

            if key:
                if canonical_querystring:
                    canonical_querystring += "&"
                canonical_querystring += u'='.join([key, val])

        return canonical_querystring

    def headers_to_sign(self, request):
        """
        Select the headers from the request that need to be included
        in the StringToSign.
        """
        header_map = HTTPHeaders()
        for name, value in request.headers.items():
            lname = name.lower()
            if lname not in SIGNED_HEADERS_BLACKLIST:
                header_map[lname] = value
        if 'host' not in header_map:
            header_map['host'] = self._canonical_host(request.url)
        return header_map

    def canonical_request(self, request):
        cr = [request.method.upper()]
        path = self.get_canonical_path(request)
        cr.append(path)
        cr.append(self.get_canonical_querystring(request))
        headers_to_sign = self.headers_to_sign(request)
        cr.append(self.canonical_headers(headers_to_sign) + '\n')
        cr.append(self.signed_headers(headers_to_sign))
        if 'X-Amz-Content-SHA256' in request.headers:
            body_checksum = request.headers['X-Amz-Content-SHA256']
        else:
            body_checksum = self.payload(request)
        cr.append(body_checksum)
        return '\n'.join(cr)

    def _canonical_host(self, url):
        url_parts = urlsplit(url)
        default_ports = {
            'http': 80,
            'https': 443
        }
        if any(url_parts.scheme == scheme and url_parts.port == port
               for scheme, port in default_ports.items()):
            # No need to include the port if it's the default port.
            return url_parts.hostname
        # Strip out auth if it's present in the netloc.
        return url_parts.netloc.rsplit('@', 1)[-1]

    def string_to_sign(self, request, canonical_request):
        """
        Return the canonical StringToSign as well as a dict
        containing the original version of all headers that
        were included in the StringToSign.
        """
        sts = ['AWS4-HMAC-SHA256']
        sts.append(self.timestamp)
        sts.append(self.credential_scope(request))
        sts.append(
            hashlib.sha256(canonical_request.encode('utf-8')).hexdigest()
        )
        return '\n'.join(sts)

    def signed_headers(self, headers_to_sign):
        hdrs = ['%s' % n.lower().strip() for n in set(headers_to_sign)]
        hdrs = sorted(hdrs)
        return ';'.join(hdrs)
    # def _normalize_url_path(self, path):
    #     normalized_path = quote(normalize_url_path(path), safe='/~')
    #     return normalized_path

    def _sign(self, key, msg, hex=False):
        if hex:
            sig = hmac.new(
                key,
                msg.encode('utf-8'),
                hashlib.sha256).hexdigest()
        else:
            sig = hmac.new(key, msg.encode('utf-8'), hashlib.sha256).digest()
        return sig

    def signature(self, string_to_sign, request):
        key = self.aws_secret_access_key
        k_date = self._sign(('AWS4' + key).encode('utf-8'),
                            self.timestamp[0:8])
        k_region = self._sign(k_date, self.aws_region)
        k_service = self._sign(k_region, self.service)
        k_signing = self._sign(k_service, 'aws4_request')
        return self._sign(k_signing, string_to_sign, hex=True)

    def credential_scope(self, request):
        scope = []
        scope.append(self.timestamp[0:8])
        scope.append(self.aws_region)
        scope.append(self.service)
        scope.append('aws4_request')
        return '/'.join(scope)

    def scope(self, request):
        scope = [self.aws_access_key]
        scope.append(self.timestamp[0:8])
        scope.append(self.aws_region)
        scope.append(self.service)
        scope.append('aws4_request')
        return '/'.join(scope)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
# import datetime
import random
import uuid

import mock

from openstackclient.tests.unit import utils
from otcextensions.tests.unit.osclient import test_base

from otcextensions.sdk.dms.v1 import group
from otcextensions.sdk.dms.v1 import instance
from otcextensions.sdk.dms.v1 import queue
from otcextensions.sdk.dms.v1 import quota
from otcextensions.sdk.dms.v1 import topic


def gen_data(data, columns):
    """Fill expected data tuple based on columns list
    """
    return tuple(getattr(data, attr, '') for attr in columns)


def gen_data_dict(data, columns):
    """Fill expected data tuple based on columns list
    """
    return tuple(data.get(attr, '') for attr in columns)


class TestDMS(utils.TestCommand):

    def setUp(self):
        super(TestDMS, self).setUp()

        self.app.client_manager.dms = mock.Mock()


class FakeQueue(test_base.Fake):
    """Fake one or more Queue"""

    @classmethod
    def generate(cls):
        object_info = {
            'name': 'group-' + uuid.uuid4().hex,
            'id': 'id-' + uuid.uuid4().hex,
            'description': 'SOME description',
            'queue_mode': random.choice(['NORMAL', 'FIFO',
                                         'KAFKA_HA', 'KAFKA_HT']),
            'redrive_policy': random.choice(['enable', 'disable']),
            'max_consume_count': random.randint(1, 100),
            'retention_hours': random.randint(1, 72)
        }
        obj = queue.Queue.existing(**object_info)
        return obj


class FakeGroup(test_base.Fake):
    """Fake one or more Group"""

    @classmethod
    def generate(cls):
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            'name': 'group-' + uuid.uuid4().hex,
            'produced_messages': random.randint(1, 100),
            'consumed_messages': random.randint(1, 100),
            'available_messages': random.randint(1, 100),
            'produced_deadletters': random.randint(1, 100),
            'available_deadletters': random.randint(1, 100),
        }
        obj = group.Group.existing(**object_info)
        return obj


class FakeQuota(test_base.Fake):
    """Fake one or more Quota"""

    @classmethod
    def generate(cls):
        object_info = {
            'type': 'type-' + uuid.uuid4().hex,
            'quota': random.randint(1, 100),
            'min': random.randint(1, 100),
            'max': random.randint(1, 100),
            'used': random.randint(1, 100),
        }
        obj = quota.Quota.existing(**object_info)
        return obj


class FakeInstance(test_base.Fake):
    @classmethod
    def generate(cls):
        object_info = {
            'name': 'name-' + uuid.uuid4().hex,
            'description': 'name-' + uuid.uuid4().hex,
            'engine_name': 'engine-' + uuid.uuid4().hex,
            'engine_version': 'ver-' + uuid.uuid4().hex,
            'storage': random.randint(1, 100),
            'access_user': 'user-' + uuid.uuid4().hex,
            'password': uuid.uuid4().hex,
            'router_id': 'router_id-' + uuid.uuid4().hex,
            'router_name': 'router_name-' + uuid.uuid4().hex,
            'network_id': 'net_id-' + uuid.uuid4().hex,
            'subnet_name': 'subnet_name-' + uuid.uuid4().hex,
            'security_group_id': 'security_group_id-' + uuid.uuid4().hex,
            'security_group_name': 'security_group_name-' + uuid.uuid4().hex,
            'availability_zones': ['az' + uuid.uuid4().hex],
            'product_id': 'product-' + uuid.uuid4().hex,
            'maintenance_begin': 'mb-' + uuid.uuid4().hex,
            'maintenance_end': 'me-' + uuid.uuid4().hex,
            'is_public': random.choice([True, False]),
            'is_ssl': random.choice([True, False]),
            'kafka_public_status': 'kps-' + uuid.uuid4().hex,
            'public_bandwidth': random.randint(1, 100),
            'retention_policy': random.choice(['produce_reject', 'time_base']),
            'storage_spec_code': random.choice(['dms.physical.storage.high',
                                                'dms.physical.storage.ultra'])
        }

        obj = instance.Instance.existing(**object_info)
        return obj


class FakeTopic(test_base.Fake):
    @classmethod
    def generate(cls):
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            'replication': random.randint(1, 3),
            'partition': random.randint(1, 21),
            'retention_time': random.randint(1, 169),
            'is_sync_replication': random.choice([True, False]),
            'is_sync_flush': random.choice([True, False]),
            'instance_id': 'iid-' + uuid.uuid4().hex
        }

        obj = topic.Topic.existing(**object_info)
        return obj



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock

from otcextensions.tests.unit.osclient.css.v1 import fakes
from otcextensions.osclient.css.v1 import certificate


class TestDownloadCertificate(fakes.TestCss):

    def setUp(self):
        super(TestDownloadCertificate, self).setUp()

        self.cmd = certificate.DownloadCertificate(self.app, None)
        self.client.download_certificate = mock.Mock(return_value=None)

    def test_download_certificate(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.assertIsNone(result)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import _log

from otcextensions.tests.functional import base

_logger = _log.setup_logging('openstack')


class TestResourceTag(base.BaseFunctionalTest):
    def setUp(self):
        super(TestResourceTag, self).setUp()
        self.tms = self.conn.tms

    def test_create_tag(self):
        attrs = {
            "project_id": "786ef11caa5c43ff80256be4c7fee8b7",
            "resources": [
                {"resource_id": "2079d0a6-3dbc-4d59-99da-6b8b7c899a97",
                 "resource_type": "vpc"}],
            "tags": [{"key": "ENV1", "value": "dev1"}],
        }
        result = self.tms.create_resource_tag(**attrs)
        self.assertNotEqual(result, None)

    def test_delete_tag(self):
        attrs = {
            "project_id": "786ef11caa5c43ff80256be4c7fee8b7",
            "resources": [
                {"resource_id": "2079d0a6-3dbc-4d59-99da-6b8b7c899a97",
                 "resource_type": "vpc"}],
            "tags": [{"key": "ENV1", "value": "dev1"}],
        }
        result = self.tms.delete_resource_tag(**attrs)
        self.assertNotEqual(result, None)

    def test_tags_list(self):
        query = {
            "resource_id": "2079d0a6-3dbc-4d59-99da-6b8b7c899a97",
            "resource_type": "vpc",
            "project_id": "786ef11caa5c43ff80256be4c7fee8b7"
        }
        tags = list(self.tms.resource_tags(**query))
        self.assertGreaterEqual(len(tags), 0)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''DeH Host v1 action implementations'''
import logging

from cliff import columns
from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _
from otcextensions.common import sdk_utils

LOG = logging.getLogger(__name__)

HOST_STATES = ['available', 'fault', 'released']


class HostPropertiesFormatter(columns.FormattableColumn):

    def human_readable(self):
        if self._value is None:
            return None
        res = self._value.to_dict()
        res.pop('available_instance_capacities', None)

        return res

    def machine_readable(self):
        if self._value is None:
            return None

        return self._value.to_dict()


_formatters = {
    'tags': sdk_utils.ListOfDictColumn,
    'host_properties': HostPropertiesFormatter
}


def _get_columns(item):
    column_map = {
    }
    invisible_columns = ['location']
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map,
                                                           invisible_columns)


class ListHost(command.Lister):
    _description = _('List DeH hosts')
    columns = (
        'id', 'name', 'auto_placement', 'availability_zone',
        'available_vcpus', 'available_memory'
    )

    def get_parser(self, prog_name):
        parser = super(ListHost, self).get_parser(prog_name)
        parser.add_argument(
            '--id',
            metavar='<id>',
            help=_('Host id.')
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Host name.')
        )
        parser.add_argument(
            '--host_type',
            metavar='<host_type>',
            help=_('Host type.')
        )
        parser.add_argument(
            '--host_type_name',
            metavar='<host_type_name>',
            help=_('Host type name.')
        )
        parser.add_argument(
            '--flavor',
            metavar='<flavor>',
            help=_('Flavor ID.')
        )
        parser.add_argument(
            '--status',
            metavar='{' + ','.join(HOST_STATES) + '}',
            type=lambda s: s.lower(),
            choices=HOST_STATES,
            help=_('Host status filter.')
        )
        parser.add_argument(
            '--tenant',
            metavar='<tenant>',
            help=_('Tenant ID or "all".')
        )
        parser.add_argument(
            '--availability_zone',
            metavar='<availability_zone>',
            help=_('Availability zone.')
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_('Number of entries to display.')
        )
        parser.add_argument(
            '--marker',
            metavar='<marker>',
            help=_('ID of the last record on the previous page.')
        )
        parser.add_argument(
            '--changes_since',
            metavar='<changes_since>',
            help=_('Filters the response by a date and time stamp when the '
                   'DeH last changed status. Format: '
                   'CCYY-MM-DDThh:mm:ss+hh:mm')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.deh

        query = {}

        if parsed_args.id:
            query['id'] = parsed_args.id
        if parsed_args.name:
            query['name'] = parsed_args.name
        if parsed_args.host_type:
            query['host_type'] = parsed_args.host_type
        if parsed_args.host_type_name:
            query['host_type_name'] = parsed_args.host_type_name
        if parsed_args.flavor:
            query['flavor'] = parsed_args.flavor
        if parsed_args.status:
            query['status'] = parsed_args.status
        if parsed_args.tenant:
            query['tenant'] = parsed_args.tenant
        if parsed_args.availability_zone:
            query['availability_zone'] = parsed_args.availability_zone
        if parsed_args.limit:
            query['limit'] = parsed_args.limit
        if parsed_args.marker:
            query['marker'] = parsed_args.marker
        if parsed_args.changes_since:
            query['changes_since'] = parsed_args.changes_since

        data = client.hosts(**query)

        table = (self.columns,
                 (utils.get_item_properties(
                     s, self.columns, formatters=_formatters
                 ) for s in data))
        return table


class ShowHost(command.ShowOne):
    _description = _('Show the DeH Host details')

    def get_parser(self, prog_name):
        parser = super(ShowHost, self).get_parser(prog_name)

        parser.add_argument(
            'host',
            metavar='<host>',
            help=_('UUID of the host.')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.deh

        obj = client.find_host(
            parsed_args.host,
        )

        obj.fetch_tags(client)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class DeleteHost(command.Command):
    _description = _('Delete host')

    def get_parser(self, prog_name):
        parser = super(DeleteHost, self).get_parser(prog_name)

        parser.add_argument(
            'host',
            metavar='<host>',
            nargs='+',
            help=_('UUID or name of the host.')
        )

        return parser

    def take_action(self, parsed_args):
        if parsed_args.host:
            client = self.app.client_manager.deh
            for host in parsed_args.host:
                client.delete_host(host=host, ignore_missing=False)


class CreateHost(command.ShowOne):
    _description = _('Create/allocate host')

    columns = ('id')

    def get_parser(self, prog_name):
        parser = super(CreateHost, self).get_parser(prog_name)

        parser.add_argument(
            '--name',
            metavar='<name>',
            required=True,
            help=_('DNS Name for the host.')
        )
        parser.add_argument('--auto_placement',
                            action='store_const',
                            default='on',
                            const='on',
                            dest='auto_placement')
        parser.add_argument('--no-auto_placement',
                            action='store_const',
                            const='off',
                            dest='auto_placement')
        parser.add_argument(
            '--availability_zone',
            metavar='<az>',
            required=True,
            help=_('The AZ the host belongs to.')
        )
        parser.add_argument(
            '--host_type',
            metavar='<type>',
            help=_('DeH type.')
        )
        parser.add_argument(
            '--quantity',
            metavar='[0..]',
            type=int,
            default=1,
            help=_('Number of DeHs to allocate.')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.deh

        attrs = {}

        if parsed_args.name:
            attrs['name'] = parsed_args.name
        if parsed_args.auto_placement:
            attrs['auto_placement'] = parsed_args.auto_placement
        if parsed_args.availability_zone:
            attrs['availability_zone'] = parsed_args.availability_zone
        if parsed_args.host_type:
            attrs['host_type'] = parsed_args.host_type
        if parsed_args.quantity:
            attrs['quantity'] = parsed_args.quantity

        obj = client.create_host(
            **attrs
        )

        table = (self.columns,
                 (utils.get_item_properties(
                     s, self.columns, formatters=_formatters
                 ) for s in obj.dedicated_host_ids))
        return table


class SetHost(command.ShowOne):
    _description = _('Update Host attributes')

    def get_parser(self, prog_name):
        parser = super(SetHost, self).get_parser(prog_name)

        parser.add_argument(
            'host',
            metavar='<host>',
            help=_('UUID or name of the host.')
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('DNS Name for the host.')
        )
        parser.add_argument(
            '--auto_placement',
            action='store_const',
            const='on',
            dest='auto_placement'
        )
        parser.add_argument(
            '--no-auto_placement',
            action='store_const',
            const='off',
            dest='auto_placement'
        )
        parser.add_argument(
            '--tag',
            metavar="<key=value>",
            action="append",
            help=_('Tag to add for this host '
                   '(repeat option to set multiple tags)')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.deh

        attrs = {}

        if parsed_args.name:
            attrs['name'] = parsed_args.name
        if parsed_args.auto_placement:
            attrs['auto_placement'] = parsed_args.auto_placement

        host = client.find_host(parsed_args.host, ignore_missing=False)

        if host:
            if attrs:
                obj = client.update_host(
                    host=host,
                    **attrs
                )

            if parsed_args.tag:
                tags = []
                for tag in parsed_args.tag:
                    (k, v) = tag.split('=')
                    xtag = {
                        'key': k,
                        'value': v
                    }
                    tags.append(xtag)

                obj = host.add_tags(client, tags)

            display_columns, columns = _get_columns(obj)
            data = utils.get_item_properties(obj, columns,
                                             formatters=_formatters)

            return (display_columns, data)


class UnsetHost(command.ShowOne):
    _description = _('Remove Host attributes')

    def get_parser(self, prog_name):
        parser = super(UnsetHost, self).get_parser(prog_name)

        parser.add_argument(
            'host',
            metavar='<host>',
            help=_('UUID or name of the host.')
        )
        parser.add_argument(
            '--tag',
            metavar="<key>",
            action="append",
            help=_('Tag key to remove from this host '
                   '(repeat option to remove multiple tags)')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.deh

        host = client.find_host(parsed_args.host, ignore_missing=False)
        host.fetch_tags(client)

        if host:
            if parsed_args.tag:
                tags = []
                for tag in parsed_args.tag:
                    tag_pair = tag.split('=')
                    k = tag_pair[0]
                    existing_tag = None
                    for tag in host.tags:
                        # Find whether we have such key
                        if tag['key'] == k:
                            existing_tag = tag
                            break

                    if existing_tag:
                        xtag = {}
                        if len(tag_pair) == 2:
                            # User provided us also a value
                            xtag = {
                                'key': k,
                                'value': tag_pair[1]
                            }
                        else:
                            # Existing tag value will be used
                            xtag = {
                                'key': k,
                                'value': existing_tag['value']
                            }

                        tags.append(xtag)

                if tags:
                    host.remove_tags(client, tags)

            display_columns, columns = _get_columns(host)
            data = utils.get_item_properties(host, columns,
                                             formatters=_formatters)

            return (display_columns, data)


class ListServer(command.Lister):
    _description = _('List Servers on a DeH')
    columns = (
        'addresses', 'id', 'name', 'metadata', 'status', 'user_id'
    )

    def get_parser(self, prog_name):
        parser = super(ListServer, self).get_parser(prog_name)

        parser.add_argument(
            'host',
            metavar='<host>',
            help=_('UUID of the DeH host.')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.deh

        host = client.find_host(parsed_args.host, ignore_missing=False)

        if host:
            data = client.servers(host=host)

            table = (self.columns,
                     (utils.get_item_properties(
                         s, self.columns, formatters=_formatters
                     ) for s in data))
            return table


class ListHostType(command.Lister):
    _description = _('List DeH host types')
    columns = (
        'host_type', 'host_type_name'
    )

    def get_parser(self, prog_name):
        parser = super(ListHostType, self).get_parser(prog_name)

        parser.add_argument(
            'az',
            metavar='<az>',
            help=_('Availability zone.')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.deh

        data = client.host_types(parsed_args.az)

        table = (self.columns,
                 (utils.get_item_properties(
                     s, self.columns, formatters=_formatters
                 ) for s in data))
        return table



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource

from otcextensions.sdk.cce.v3 import _base


class StatusSpec(resource.Resource):
    # Properties
    #: Cluster status.
    status = resource.Body('phase')
    #: Current node count.
    current_node = resource.Body('currentNode')


class VolumeExtendParams(resource.Resource):
    # Properties
    # Usage mode of data disk e.g. docker
    use_type = resource.Body('useType')


class DataVolumeEncryptionSpec(resource.Resource):
    # Properties
    # Whether an EVS disk is encrypted (1) or not (0).
    encrypted = resource.Body('__system__encrypted', type=bool, default=False)
    # CMK ID used for encryption.
    cmk_id = resource.Body('__system__cmkid')


class VolumeSpec(resource.Resource):
    # Properties
    # Disk Size in GB.
    size = resource.Body('size', type=int)
    # Volume type: [SATA, SAS, SSD].
    type = resource.Body('volumetype')


class DataVolumeSpec(VolumeSpec):
    # Properties
    # Extended fields in dictionary format
    extend_params = resource.Body('extendParam', type=VolumeExtendParams)
    # Mandatory data if data encryption of the data Volume is necessary.
    # If data disks are created using a data disk image, this parameter
    # cannot be used.
    metadata = resource.Body('metadata', type=DataVolumeEncryptionSpec)


class PublicIPSpec(resource.Resource):
    # Properties:
    # List of IDs for the existing floating ips.
    ids = resource.Body('ids')
    # Count of the IP addresses to be dynamically created.
    count = resource.Body('count', type=int)
    # Elastic IP address. Dict of {
    #     type,
    #     bandwidth:{
    #        chargemode, size, sharetype
    #     }
    # }.
    floating_ip_spec = resource.Body('eip', type=dict)


class TagSpec(resource.Resource):
    # Properties:
    # Key of the tag
    key = resource.Body('key')
    # Value of the tag
    value = resource.Body('value')


class TaintSpec(resource.Resource):
    # Properties:
    # A key can contain letters, digits, hyphens, underscores and periods
    # up to 63 characters starting with letter or digit.
    key = resource.Body('key')
    # A value can contain letters, digits, hyphens, underscores and periods
    # up to 63 characters starting with letter or digit.
    value = resource.Body('value')
    # Effect available options: NoSchedule, PreferNoSchedule, NoExecute
    effect = resource.Body('effect')


class ExtendParamSpec(resource.Resource):
    # Properties:
    # Script required before the installation
    # Input must be Base64 encoded
    preinstall_script = resource.Body('alpha.cce/preInstall')
    # Script required after the installation
    # Input must be Base64 encoded
    postinstall_script = resource.Body('alpha.cce/postInstall')
    # ConfigMap of the Docker data disk
    lvm_config = resource.Body('DockerLVMConfigOverride')
    # Maximum number of pods on the node
    max_pods = resource.Body('maxPods', type=int)
    # ID of a custom node image
    # Mandatory if custom image is used on a bare metall node
    node_image_id = resource.Body('alpha.cce/NodeImageID')
    # Public key of the node.
    public_key = resource.Body('publicKey')


class AutoScalingSpec(resource.Resource):
    # Enable or disable auto scaling
    enable = resource.Body('enable', type=bool)
    # Minimum number of nodes after a scale-up if auto-scaling is enabled.
    min_node_count = resource.Body('minNodeCount', type=int)
    # Maximum number of nodes after scale-up, if auto-scaling is enabled.
    # The value must be equal or greater than min_node_count and cannot
    # exceed the maximum number of nodes of the cluster.
    max_node_count = resource.Body('maxNodeCount', type=int)
    # Node pool weight. A higher weight indicates a higher priority
    # in scale-up.
    priority = resource.Body('priority', type=int)
    # Interval in minutes during which nodes added after a scale-up
    # will not be deleted.
    scale_down_cooldown_time = resource.Body('scaleDownCooldownTime', type=int)


class NodeManagementSpec(resource.Resource):
    # ECS group id of the ECS group to which those nodes belong
    # after creation.
    ecs_group = resource.Body('serverGroupReference')


class NetworkIdSpec(resource.Resource):
    # ID of the network to which the NIC belongs
    network_id = resource.Body('subnetId')


class NodeNicSpec(resource.Resource):
    # Description about the primary NIC
    primary_nic = resource.Body('primaryNic', type=NetworkIdSpec)


class LoginSpec(resource.Resource):
    # SSH public key for login purposes
    ssh_key = resource.Body('sshKey')


class NodeTemplateSpec(resource.Resource):
    # Properties
    # Name of the AZ where the node resides.
    # If availability_zone is set to: random, nodes can be created in any AZ
    availability_zone = resource.Body('az')
    # Billing mode of a node. Currently, only pay-per-use is supported.
    billing_mode = resource.Body('billingMode', type=int, default=0)
    # Number of nodes.
    count = resource.Body('count', type=int)
    # Data disk parameters of a node. At present, only one data
    # disk can be configured
    data_volumes = resource.Body('dataVolumes', type=list,
                                 list_type=DataVolumeSpec)
    # ID of the ECS group where the CCE node can belong to
    ecs_group = resource.Body('ecsGroupId')
    # Extended parameters in key-value format
    extend_params = resource.Body('extendParam', type=ExtendParamSpec)
    # Flavor (mandatory)
    flavor = resource.Body('flavor')
    # Elastic IP address parameters of a node.
    floating_ip = resource.Body('publicIP', type=PublicIPSpec)
    # Kubernetes tags
    k8s_tags = resource.Body('k8sTags', type=dict)
    # Parameters for logging in to the node.
    login = resource.Body('login', type=LoginSpec)
    node_nic_spec = resource.Body('nodeNicSpec', type=NodeNicSpec)
    # Operating System of the node. EulerOS and CentOS (K8s version >= 1.17)
    # are supported.
    os = resource.Body('os')
    # System disk parameters of the node.
    root_volume = resource.Body('rootVolume', type=VolumeSpec)
    # Tags of a Node
    tags = resource.Body('userTags', type=list, list_type=TagSpec)
    # Taints are used to configure anti-affinity
    taints = resource.Body('taints', type=list, list_type=TaintSpec)


class NodePoolSpec(resource.Resource):
    # Autoscaling parameters
    autoscaling = resource.Body('autoscaling', type=AutoScalingSpec)
    # Expected number of nodes in this node pool.
    initial_node_count = resource.Body('initialNodeCount', type=int)
    node_management = resource.Body('nodeManagement', type=NodeManagementSpec)
    # Node pool type. Currently only ECSs are supported
    # Value: vm
    node_pool_type = resource.Body('type')
    # Template of the node specification.
    node_template_spec = resource.Body('nodeTemplate', type=NodeTemplateSpec)


class MetaDataSpec(resource.Resource):
    # Name of the node pool
    name = resource.Body('name')
    #: UUID
    #: *Type:str
    id = resource.Body('uid', alternate_id=True)


class NodePool(_base.Resource):

    base_path = '/clusters/%(cluster_id)s/nodepools'

    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    # Properties
    # Cluster id.
    cluster_id = resource.URI('cluster_id')
    # Spec
    spec = resource.Body('spec', type=NodePoolSpec)
    # other metadata
    metadata = resource.Body('metadata', type=MetaDataSpec)
    #: Cluster status
    status = resource.Body('status', type=StatusSpec)

    @classmethod
    def new(cls, **kwargs):
        if 'kind' not in kwargs:
            kwargs['kind'] = 'NodePool'
        if 'apiVersion' not in kwargs:
            kwargs['apiVersion'] = 'v3'
        return cls(_synchronized=False, **kwargs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from keystoneauth1 import adapter

import copy
import mock

from openstack import resource

from openstack.tests.unit import base

from otcextensions.sdk.rds.v3 import configuration


IDENTIFIER = 'ID'
EXAMPLE = {
    "id": IDENTIFIER,
    "name": "paramsGroup-b6d2",
    "description": "",
    "datastore": {"type": "PostgreSQL"},
    "datastore_version_name": "2016_SE",
    "datastore_name": "sqlserver",
    "created": "2018-09-15T03:33:07+0800",
    "updated": "2018-09-15T03:33:07+0800",
    "user_defined": True,
    "vcpus": "1",
    "ram": 2,
    "spec_code": "rds.mysql.c2.medium.ha",
    "instance_mode": "ha",
    "values": {
        "max_connections": "10",
        "autocommit": "OFF"
    }
}


class TestConfiguration(base.TestCase):

    def setUp(self):
        super(TestConfiguration, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.put = mock.Mock()

    def test_basic(self):
        sot = configuration.Configuration()

        self.assertEqual('/configurations', sot.base_path)
        self.assertEqual('configurations', sot.resources_key)
        self.assertEqual('configuration', sot.resource_key)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)

    def test_make_it(self):
        sot = configuration.Configuration(**EXAMPLE)
        self.assertEqual(IDENTIFIER, sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['datastore'], sot.datastore)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['datastore_name'], sot.datastore_name)
        self.assertEqual(EXAMPLE['datastore_version_name'],
                         sot.datastore_version_name)
        self.assertEqual(EXAMPLE['created'], sot.created_at)
        self.assertEqual(EXAMPLE['updated'], sot.updated_at)
        self.assertEqual(EXAMPLE['user_defined'], sot.is_user_defined)
        self.assertEqual(EXAMPLE['values'], sot.values)

    def test_apply(self):
        sot = configuration.Configuration(id=IDENTIFIER)
        instances = ['id1', 'id2']

        resp = mock.Mock()
        resp.body = {
            "configuration_id": IDENTIFIER,
            "configuration_name": "paramsGroup-bcf9",
            "apply_results": [{
                "instance_id": "fe5f5a07539c431181fc78220713aebein01",
                "instance_name": "zyy1",
                "restart_required": False,
                "success": False
            }, {
                "instance_id": "73ea2bf70c73497f89ee0ad4ee008aa2in01",
                "instance_name": "zyy2",
                "restart_required": False,
                "success": False
            }],
            "success": False
        }
        resp.json = mock.Mock(return_value=copy.deepcopy(resp.body))
        resp.headers = {}
        resp.status_code = 200
        self.sess.put.return_value = resp

        updated = sot.apply(self.sess, instances)
        self.sess.put.assert_called_with(
            'configurations/ID/apply',
            json={'instance_ids': ['id1', 'id2']}
        )

        self.assertEqual(resp.body['configuration_id'], updated.id)
        self.assertEqual(resp.body['configuration_name'], updated.name)
        self.assertEqual(resp.body['apply_results'],
                         sot.apply_results)

    def test_create(self):
        sot = configuration.Configuration(id=IDENTIFIER)

        with mock.patch.object(resource.Resource, 'create') as res_mock:
            sot.create(self.sess)
            res_mock.assert_called_with(self.sess,
                                        prepend_key=False,
                                        base_path=None)

    def test_commit(self):
        sot = configuration.Configuration(id=IDENTIFIER)

        with mock.patch.object(resource.Resource, 'commit') as res_mock:
            sot.commit(self.sess)
            res_mock.assert_called_with(self.sess,
                                        prepend_key=False)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Retry a task or terminate the impact of a task
"""

import openstack

openstack.enable_logging(True)
conn = openstack.connect()

cluster_name_or_id = '3b300b4e-2aa9-45c0-b898-a9e6fa319922'
job_id = '0249620d-1c4a-4211-943a-ced7b9a3cda5'
retry_mode = 'abort'

cluster = conn.css.find_cluster(cluster_name_or_id)

conn.css.retry_cluster_upgrade_job(cluster, job_id, retry_mode)
